<?php
/*+**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 ************************************************************************************/
require_once('data/CRMEntity.php');
require_once('data/Tracker.php');
require_once 'vtlib/Vtiger/Module.php';

class ModCommentsCore extends CRMEntity {
	var $db, $log; // Used in class functions of CRMEntity

	var $table_name;
	var $table_index= 'modcommentsid';

	/** Indicator if this is a custom module or standard module */
	var $IsCustomModule = true;

	/**
	 * Mandatory table for supporting custom fields.
	 */
	var $customFieldTable = Array();

	/**
	 * Mandatory for Saving, Include tables related to this module.
	 */
	var $tab_name = Array();

	/**
	 * Mandatory for Saving, Include tablename and tablekey columnname here.
	 */
	var $tab_name_index = Array();

	/**
	 * Mandatory for Listing (Related listview)
	 */
	var $list_fields = Array (
		/* Format: Field Label => Array(tablename, columnname) */
		// tablename should not have prefix 'vtiger_'
		'Comment' => Array('modcomments', 'commentcontent'),
		'Assigned To' => Array('crmentity','smownerid')
	);
	var $list_fields_name = Array (
		/* Format: Field Label => fieldname */
		'Comment' => 'commentcontent',
		'Assigned To' => 'assigned_user_id'
	);

	// Make the field link to detail view 
	var $list_link_field = 'commentcontent';

	// For Popup listview and UI type support
	var $search_fields = Array(
		/* Format: Field Label => Array(tablename, columnname) */
		// tablename should not have prefix 'vtiger_'
		'Comment' => Array('modcomments', 'commentcontent')
	);
	var $search_fields_name = Array (
		/* Format: Field Label => fieldname */
		'Comment' => 'commentcontent'
	);

	// For Popup window record selection
	var $popup_fields = Array ('commentcontent');

	// Allow sorting on the following (field column names)
	var $sortby_fields = Array ('commentcontent');

	// Should contain field labels
	//var $detailview_links = Array ('Comment');

	// For Alphabetical search
	var $def_basicsearch_col = 'commentcontent';

	// Column value to use on detail view record text display
	var $def_detailview_recname = 'commentcontent';

	// Required Information for enabling Import feature
	var $required_fields = Array ('assigned_user_id'=>1);

	// Callback function list during Importing
	var $special_functions = Array('set_import_assigned_user');

	var $default_order_by = 'commentcontent';
	var $default_sort_order='ASC';
	
	// Used when enabling/disabling the mandatory fields for the module.
	// Refers to vtiger_field.fieldname values.
	var $mandatory_fields = Array('createdtime', 'modifiedtime', 'commentcontent');
	
	//crmv@10759
	var $search_base_field = 'commentcontent';
	//crmv@10759 e
	
	function __construct() {
		global $log, $currentModule, $table_prefix;
		parent::__construct(); // crmv@37004
		$this->table_name = $table_prefix.'_modcomments';
		$this->tab_name_index = Array(
				$table_prefix.'_crmentity' => 'crmid',
				$table_prefix.'_modcomments' => 'modcommentsid',
				$table_prefix.'_modcommentscf'=>'modcommentsid');
		$this->tab_name = Array($table_prefix.'_crmentity',$table_prefix.'_modcomments', $table_prefix.'_modcommentscf');
		$this->customFieldTable = Array($table_prefix.'_modcommentscf', 'modcommentsid');
		$this->column_fields = getColumnFields($currentModule);
		$this->db = PearDatabase::getInstance();
		$this->log = $log;
	}

	function getSortOrder()
	{
		global $log,$currentModule;
		$log->debug("Entering getSortOrder() method ...");
		$use_default_order_by = '';
		//default listview sorting
		if(PerformancePrefs::getBoolean('LISTVIEW_DEFAULT_SORTING', true)) {
				$use_default_sort_order = $this->default_sort_order;
		}
		//crmv default listview customview sorting
		if ($this->customview_sort_order != '' && $use_default_sort_order != $this->customview_sort_order)
				$use_default_sort_order = $this->customview_sort_order;		
		if(isset($_REQUEST['sorder']))
			$sorder = $this->db->sql_escape_string($_REQUEST['sorder']);
		elseif ($_REQUEST['override_orderby'] == 'true')
			$sorder = $use_default_sort_order;
		else
			$sorder = (($_SESSION[$currentModule.'_SORT_ORDER'] != '')?($_SESSION[$currentModule.'_SORT_ORDER']):($use_default_sort_order));

		$log->debug("Exiting getSortOrder method ...");
		return $sorder;
	}

	function getOrderBy()
	{
		global $log,$currentModule;
		$log->debug("Entering getOrderBy() method ...");
		$use_default_order_by = '';
		//default listview sorting
		if(PerformancePrefs::getBoolean('LISTVIEW_DEFAULT_SORTING', true)) {
				$use_default_order_by = $this->default_order_by;
		}
		//crmv default listview customview sorting
		if ($this->customview_order_by != '' && $use_default_order_by != $this->customview_order_by)
				$use_default_order_by = $this->customview_order_by;
		if (isset($_REQUEST['order_by']))
			$order_by = $this->db->sql_escape_string($_REQUEST['order_by']);
		elseif ($_REQUEST['override_orderby'] == 'true')
			$order_by = $use_default_order_by;
		else	
			$order_by = (($_SESSION[$currentModule.'_ORDER_BY'] != '')?($_SESSION[$currentModule.'_ORDER_BY']):($use_default_order_by));

		$log->debug("Exiting getOrderBy method ...");
		return $order_by;
	}
	
	function save_module($module) {
		global $current_user, $adb,$table_prefix;
		require_once('modules/SDK/src/Notifications/Notifications.php');
		
		//crmv@2963m
		if (getSalesEntityType($this->column_fields['related_to']) == 'Messages') {
			$result = $adb->pquery("SELECT messagehash FROM {$table_prefix}_messages WHERE messagesid = ?",array($this->column_fields['related_to']));
			if ($result && $adb->num_rows($result) > 0) {
				$messagehash = $adb->query_result($result,0,'messagehash');
				if ($adb->isMysql()) {
					$adb->pquery("insert ignore into {$table_prefix}_messagesrel (messagehash,crmid,module) values (?,?,?)",array($messagehash,$this->id,'ModComments'));
				} else {
					$result = $adb->pquery("select * from {$table_prefix}_messagesrel where messagehash = ? and crmid = ?",array($messagehash,$this->id));
					if (!$result || $adb->num_rows($result) == 0) {
						$adb->pquery("insert into {$table_prefix}_messagesrel (messagehash,crmid,module) values (?,?,?)",array($messagehash,$this->id,'ModComments'));
					}
				}
			}
		}
		//crmv@2963me
		
		if (!in_array($this->column_fields['parent_comments'],array('',0))) {
			//reply

			$focusFather = CRMEntity::getInstance('ModComments');
			$focusFather->retrieve_entity_info($this->column_fields['parent_comments'],'ModComments');
			
			$users_to_notify = array();
			if ($focusFather->column_fields['visibility_comm'] == 'All') {
				/*
				//notifico a ( (assegnatario commento padre + utenti che hanno risp almeno una volta) - autore corrente )
				$result = $adb->pquery('SELECT smownerid FROM '.$table_prefix.'_modcomments INNER JOIN '.$table_prefix.'_crmentity ON '.$table_prefix.'_crmentity.crmid = '.$table_prefix.'_modcomments.modcommentsid WHERE deleted = 0 AND parent_comments = ?',array($this->column_fields['parent_comments']));
				if ($result && $adb->num_rows($result) > 0) {
					while($row=$adb->fetchByAssoc($result)) {
						if (!in_array($row['smownerid'],$users_to_notify)) {
							$users_to_notify[] = $row['smownerid'];
						}
					}
				}
				*/
				//notifico a (tutti gli utenti attivi e abilitati alle conversazioni pubbliche - autore corrente)
				$result = $adb->pquery('select id from '.$table_prefix.'_users where deleted = 0 and status = ? and receive_public_talks = ?',array('Active','1'));
				if ($result && $adb->num_rows($result) > 0) {
					while($row = $adb->fetchByAssoc($result)) {
						$users_to_notify[] = $row['id'];
					}
				}
			} elseif ($focusFather->column_fields['visibility_comm'] == 'Users') {
				//notifico a ( (assegnatario commento padre + utenti invitati) - autore corrente )
				$users_to_notify[] = $focusFather->column_fields['assigned_user_id'];
				$column = 'user';
				$adb->format_columns($column);
				$result = $adb->pquery('SELECT '.$column.' FROM '.$table_prefix.'_modcomments_users WHERE id = ?',array($this->column_fields['parent_comments']));
				if ($result && $adb->num_rows($result) > 0) {
					while($row=$adb->fetchByAssoc($result)) {
						$users_to_notify[] = $row['user'];
					}
				}
			}
			$users_to_notify = array_filter($users_to_notify);
			foreach($users_to_notify as $user) {
				if ($user == $this->column_fields['assigned_user_id']) {
					continue;
				}
				$notifications = new Notifications($user,'ModComments');
				$notifications->addNotification($this->id);
			}
		} elseif ($this->column_fields['visibility_comm'] == 'Users' && isset($_REQUEST['users_comm'])) {
			//new father comment to specified users
			
			$adb->pquery('delete from '.$table_prefix.'_modcomments_users where id = ?',array($this->id));
			$idlist = vtlib_purify($_REQUEST['users_comm']);
			if ($idlist != '') {
				$idlist = array_filter(explode('|',$idlist));
				$column = 'user';
				$adb->format_columns($column);
				foreach($idlist as $id) {
					$adb->pquery('insert into '.$table_prefix.'_modcomments_users (id,'.$column.') values (?,?)',array($this->id,$id));
					//notifico gli utenti invitati
					$notifications = new Notifications($id,'ModComments');
					$notifications->addNotification($this->id);
				}
			}
		} elseif ($this->column_fields['visibility_comm'] == 'All') {
			//new father comment to all
			
			//notifico a (tutti gli utenti attivi e abilitati alle conversazioni pubbliche - autore corrente)
			$result = $adb->pquery('select id from '.$table_prefix.'_users where deleted = 0 and status = ? and receive_public_talks = ?',array('Active','1'));
			if ($result && $adb->num_rows($result) > 0) {
				while($row = $adb->fetchByAssoc($result)) {
					if ($row['id'] == $this->column_fields['assigned_user_id']) {
						continue;
					}
					$notifications = new Notifications($row['id'],'ModComments');
					$notifications->addNotification($this->id);
				}
			}
		}
	}

	// crmv@43050 - add users to the current
	function addUsers($userlist = array()) {
		global $adb, $table_prefix;
		require_once('modules/SDK/src/Notifications/Notifications.php');

		$commentid = $this->id;

		if (!is_array($userlist)) $userlist = array($userlist);

		if (count($userlist) == 0 || $this->column_fields['visibility_comm'] != 'Users') return false;

		$column = 'user';
		$adb->format_columns($column);

		$addedUsers = false;
		foreach ($userlist as $userid) {
			// check if present
			$res = $adb->pquery("select id from {$table_prefix}_modcomments_users where id = ? and $column = ?", array($commentid,$userid));
			if ($res && $adb->num_rows($res) == 0) {
				// if not, add and notify
				$adb->pquery("insert into {$table_prefix}_modcomments_users (id,$column) values (?,?)", array($commentid,$userid));

				// notifico gli utenti invitati
				$notifications = new Notifications($userid,'ModComments');
				$notifications->addNotification($commentid);
				$addedUsers = true;
			}
		}
		if ($addedUsers) {
			// update modified time
			$adb->pquery("update {$table_prefix}_crmentity set modifiedtime = ? where crmid = ?", array(date('Y-m-d H:i:s'), $commentid));
		}
		return true;
	}
	// crmv@43050e

	/**
	 * Return query to use based on given modulename, fieldname
	 * Useful to handle specific case handling for Popup
	 */
	function getQueryByModuleField($module, $fieldname, $srcrecord) {
		// $srcrecord could be empty
	}

	/**
	 * Get list view query (send more WHERE clause condition if required)
	 */
	/**
	 * Get list view query (send more WHERE clause condition if required)
	 */
	function getListQuery($module, $where='', $replies=false, $skip_parent_join=false,$joinusers = false) {	//crmv@32429
		global $table_prefix;
		$query = "SELECT * ";
		
		// Keep track of tables joined to avoid duplicates
		$joinedTables = array();

		// Select Custom Field Table Columns if present
//		if(!empty($this->customFieldTable)) $query .= ", " . $this->customFieldTable[0] . ".* ";

		$query .= " FROM $this->table_name";

		$query .= "	INNER JOIN ".$table_prefix."_crmentity ON ".$table_prefix."_crmentity.crmid = $this->table_name.$this->table_index";

		$joinedTables[] = $this->table_name;
		$joinedTables[] = $table_prefix.'_crmentity';
		
		// Consider custom table join as well.
		if(!empty($this->customFieldTable)) {
			$query .= " INNER JOIN ".$this->customFieldTable[0]." ON ".$this->customFieldTable[0].'.'.$this->customFieldTable[1] .
				      " = $this->table_name.$this->table_index";
			$joinedTables[] = $this->customFieldTable[0];
		}
		if (!$replies && !$joinusers) {
			$query .= " LEFT JOIN ".$table_prefix."_users ON ".$table_prefix."_users.id = ".$table_prefix."_crmentity.smownerid";
			$query .= " LEFT JOIN ".$table_prefix."_groups ON ".$table_prefix."_groups.groupid = ".$table_prefix."_crmentity.smownerid";
			$joinedTables[] = $table_prefix.'_users';
			$joinedTables[] = $table_prefix.'_groups';
		}

		if (!$skip_parent_join) {	//crmv@32429
			$linkedModulesQuery = $this->db->pquery("SELECT distinct fieldname, columnname, relmodule FROM ".$table_prefix."_field" .
					" INNER JOIN ".$table_prefix."_fieldmodulerel ON ".$table_prefix."_fieldmodulerel.fieldid = ".$table_prefix."_field.fieldid" .
					" WHERE uitype='10' AND ".$table_prefix."_fieldmodulerel.module=?", array($module));
			$linkedFieldsCount = $this->db->num_rows($linkedModulesQuery);
			
			for($i=0; $i<$linkedFieldsCount; $i++) {
				$related_module = $this->db->query_result($linkedModulesQuery, $i, 'relmodule');
				$fieldname = $this->db->query_result($linkedModulesQuery, $i, 'fieldname');
				$columnname = $this->db->query_result($linkedModulesQuery, $i, 'columnname');
				
				$other =  CRMEntity::getInstance($related_module);
				vtlib_setup_modulevars($related_module, $other);
				
				if(!in_array($other->table_name, $joinedTables)) {
					$query .= " LEFT JOIN $other->table_name ON $other->table_name.$other->table_index = $this->table_name.$columnname";
					$joinedTables[] = $other->table_name;
				}
			}
		}	//crmv@32429

		global $current_user;
		if (!$replies) {
			$query .= $this->getNonAdminAccessControlQuery($module,$current_user);
		}
		$query .= "	WHERE ".$table_prefix."_crmentity.deleted = 0 ".$where;
		if (!$replies) {
			$query = $this->listQueryNonAdminChange($query, $module);	
		}
		return $query;
	}
	
	public function listQueryNonAdminChange($query,$module,$scope='') {
		global $current_user,$adb,$table_prefix;
		$query = parent::listQueryNonAdminChange($query,$module,$scope);
		if ($_REQUEST['ajxaction'] != 'LOADRELATEDLIST') {
			$query .= " AND ($this->table_name.parent_comments IS NULL OR $this->table_name.parent_comments = '')";
			$query .= " AND (
							".$table_prefix."_crmentity.smcreatorid = $current_user->id";
			if ($current_user->column_fields['receive_public_talks'] == '1') {
				$query .= "	OR ".$table_prefix."_modcomments.visibility_comm = 'All'";
			}
			$column = 'user';
			$adb->format_columns($column);
			$query .= "		OR (".$table_prefix."_modcomments.visibility_comm = 'Users' AND EXISTS(SELECT * FROM ".$table_prefix."_modcomments_users WHERE ".$table_prefix."_modcomments_users.id = ".$table_prefix."_modcomments.modcommentsid AND ".$table_prefix."_modcomments_users.$column = $current_user->id))
						)";
		}
		return $query;
	}

	/**
	 * Create query to export the records.
	 */
	function create_export_query($where,$oCustomView,$viewId)	//crmv@31775
	{
		global $current_user,$table_prefix;
		$thismodule = $_REQUEST['module'];
		
		include("include/utils/ExportUtils.php");

		//To get the Permitted fields query and the permitted fields list
		$sql = getPermittedFieldsQuery($thismodule, "detail_view");
		
		$fields_list = getFieldsListFromQuery($sql);

		$query = "SELECT $fields_list, ".$table_prefix."_users.user_name AS user_name 
					FROM ".$table_prefix."_crmentity INNER JOIN $this->table_name ON ".$table_prefix."_crmentity.crmid=$this->table_name.$this->table_index";

		if(!empty($this->customFieldTable)) {
			$query .= " INNER JOIN ".$this->customFieldTable[0]." ON ".$this->customFieldTable[0].'.'.$this->customFieldTable[1] .
				      " = $this->table_name.$this->table_index"; 
		}

		$query .= " LEFT JOIN ".$table_prefix."_groups ON ".$table_prefix."_groups.groupid = ".$table_prefix."_crmentity.smownerid";
		$query .= " LEFT JOIN ".$table_prefix."_users ON ".$table_prefix."_crmentity.smownerid = ".$table_prefix."_users.id and ".$table_prefix."_users.status='Active'";
		
		$linkedModulesQuery = $this->db->pquery("SELECT distinct fieldname, columnname, relmodule FROM ".$table_prefix."_field" .
				" INNER JOIN ".$table_prefix."_fieldmodulerel ON ".$table_prefix."_fieldmodulerel.fieldid = ".$table_prefix."_field.fieldid" .
				" WHERE uitype='10' AND ".$table_prefix."_fieldmodulerel.module=?", array($thismodule));
		$linkedFieldsCount = $this->db->num_rows($linkedModulesQuery);

		for($i=0; $i<$linkedFieldsCount; $i++) {
			$related_module = $this->db->query_result($linkedModulesQuery, $i, 'relmodule');
			$fieldname = $this->db->query_result($linkedModulesQuery, $i, 'fieldname');
			$columnname = $this->db->query_result($linkedModulesQuery, $i, 'columnname');
			
			$other = CRMEntity::getInstance($related_module);
			vtlib_setup_modulevars($related_module, $other);
			
			$query .= " LEFT JOIN $other->table_name ON $other->table_name.$other->table_index = $this->table_name.$columnname";
		}
		
		//crmv@31775
		$reportFilter = $oCustomView->getReportFilter($viewId);
		if ($reportFilter) {
			$tableNameTmp = $oCustomView->getReportFilterTableName($reportFilter,$current_user->id);
			$query .= " INNER JOIN $tableNameTmp ON $tableNameTmp.id = {$table_prefix}_crmentity.crmid";
		}
		//crmv@31775e

		$where_auto = " ".$table_prefix."_crmentity.deleted=0";

		if($where != '') $query .= " WHERE ($where) AND $where_auto";
		else $query .= " WHERE $where_auto";

		require('user_privileges/requireUserPrivileges.php'); // crmv@39110
		require('user_privileges/sharing_privileges_'.$current_user->id.'.php');

		// Security Check for Field Access
		if($is_admin==false && $profileGlobalPermission[1] == 1 && $profileGlobalPermission[2] == 1 && $defaultOrgSharingPermission[7] == 3)
		{
			//Added security check to get the permitted records only
			$query = $query." ".getListViewSecurityParameter($thismodule);
		}
		return $query;
	}

	/**
	 * Initialize this instance for importing.
	 */
	function initImport($module) {
		$this->db = PearDatabase::getInstance();
		$this->initImportableFields($module);
	}

	/**
	 * Create list query to be shown at the last step of the import.
	 * Called From: modules/Import/UserLastImport.php
	 */
	function create_import_query($module) {
		global $current_user,$table_prefix;
		$query = "SELECT ".$table_prefix."_crmentity.crmid, case when (".$table_prefix."_users.user_name is not null) then ".$table_prefix."_users.user_name else ".$table_prefix."_groups.groupname end as user_name, $this->table_name.* FROM $this->table_name
			INNER JOIN ".$table_prefix."_crmentity ON ".$table_prefix."_crmentity.crmid = $this->table_name.$this->table_index
			LEFT JOIN ".$table_prefix."_users_last_import ON ".$table_prefix."_users_last_import.bean_id=".$table_prefix."_crmentity.crmid
			LEFT JOIN ".$table_prefix."_users ON ".$table_prefix."_users.id = ".$table_prefix."_crmentity.smownerid
			LEFT JOIN ".$table_prefix."_groups ON ".$table_prefix."_groups.groupid = ".$table_prefix."_crmentity.smownerid
			WHERE ".$table_prefix."_users_last_import.assigned_user_id='$current_user->id'
			AND ".$table_prefix."_users_last_import.bean_type='$module'
			AND ".$table_prefix."_users_last_import.deleted=0";
		return $query;
	}

	function generateReportsQuery($module){
		global $current_user,$table_prefix;
		//crmv@21249
		$query = "from ".$table_prefix."_modcomments inner join ".$table_prefix."_modcommentscf ".$table_prefix."_modcommentscf
			on ".$table_prefix."_modcommentscf.modcommentsid=".$table_prefix."_modcomments.modcommentsid
			inner join ".$table_prefix."_crmentity on ".$table_prefix."_crmentity.crmid=".$table_prefix."_modcomments.modcommentsid
			left join ".$table_prefix."_groups ".$table_prefix."_groupsModComments on ".$table_prefix."_groupsModComments.groupid = ".$table_prefix."_crmentity.smownerid
			left join ".$table_prefix."_users ".$table_prefix."_usersModComments on ".$table_prefix."_usersModComments.id = ".$table_prefix."_crmentity.smownerid
			left join ".$table_prefix."_crmentity ".substr("".$table_prefix."_crmentityRelModComments",0,29)." on ".substr("".$table_prefix."_crmentityRelModComments",0,29).".crmid = ".$table_prefix."_modcomments.related_to
			and ".$table_prefix."_crmentityRelModComments.deleted=0
			left join ".$table_prefix."_leaddetails ".substr("".$table_prefix."_leaddetailsRelModComments",0,29)." on ".substr("".$table_prefix."_leaddetailsRelModComments",0,29).".leadid = ".substr("".$table_prefix."_crmentityRelModComments",0,29).".crmid
			left join ".$table_prefix."_contactdetails ".substr("".$table_prefix."_contactdetailsRelModComments",0,29)." on ".substr("".$table_prefix."_contactdetailsRelModComments",0,29).".contactid = ".substr("".$table_prefix."_crmentityRelModComments",0,29).".crmid
			left join ".$table_prefix."_account ".$table_prefix."_accountRelModComments on ".$table_prefix."_accountRelModComments.accountid = ".substr("".$table_prefix."_crmentityRelModComments",0,29).".crmid";
		$projectModule = Vtiger_Module::getInstance('Project');
		if($projectModule !== false) {
			$query .= " left join ".$table_prefix."_projecttask as ".substr("".$table_prefix."_projecttaskRelModComments",0,29)." on ".substr("".$table_prefix."_projecttaskRelModComments",0,29).".projecttaskid = ".substr("".$table_prefix."_crmentityRelModComments",0,29).".crmid
			left join ".$table_prefix."_project as ".$table_prefix."_projectRelModComments on ".$table_prefix."_projectRelModComments.projectid = ".substr("".$table_prefix."_crmentityRelModComments",0,29).".crmid";
		}
		//crmv@21249e
		return $query;
	}
	/**
	 * Delete the last imported records.
	 */
	function undo_import($module, $user_id) {
		global $adb,$table_prefix;
		$count = 0;
		$query1 = "select bean_id from ".$table_prefix."_users_last_import where assigned_user_id=? AND bean_type='$module' AND deleted=0";
		$result1 = $adb->pquery($query1, array($user_id)) or die("Error getting last import for undo: ".mysql_error()); 
		while ( $row1 = $adb->fetchByAssoc($result1))
		{
			$query2 = "update ".$table_prefix."_crmentity set deleted=1 where crmid=?";
			$result2 = $adb->pquery($query2, array($row1['bean_id'])) or die("Error undoing last import: ".mysql_error()); 
			$count++;			
		}
		return $count;
	}
	
	/**
	 * Transform the value while exporting (if required)
	 */
	function transform_export_value($key, $value) {
		return parent::transform_export_value($key, $value);
	}

	/**
	 * Function which will set the assigned user id for import record.
	 */
	function set_import_assigned_user()
	{
		global $current_user, $adb,$table_prefix;
		$record_user = $this->column_fields["assigned_user_id"];
		
		if($record_user != $current_user->id){
			$sqlresult = $adb->pquery("select id from ".$table_prefix."_users where id = ? union select groupid as id from ".$table_prefix."_groups where groupid = ?", array($record_user, $record_user));
			if($this->db->num_rows($sqlresult)!= 1) {
				$this->column_fields["assigned_user_id"] = $current_user->id;
			} else {			
				$row = $adb->fetchByAssoc($sqlresult, -1, false);
				if (isset($row['id']) && $row['id'] != -1) {
					$this->column_fields["assigned_user_id"] = $row['id'];
				} else {
					$this->column_fields["assigned_user_id"] = $current_user->id;
				}
			}
		}
	}
	
	/** 
	 * Function which will give the basic query to find duplicates
	 */
	function getDuplicatesQuery($module,$table_cols,$field_values,$ui_type_arr,$select_cols='') {
		global $table_prefix;
		$select_clause = "SELECT ". $this->table_name .".".$this->table_index ." AS recordid, ".$table_prefix."_users_last_import.deleted,".$table_cols;

		// Select Custom Field Table Columns if present
		if(isset($this->customFieldTable)) $query .= ", " . $this->customFieldTable[0] . ".* ";

		$from_clause = " FROM $this->table_name";

		$from_clause .= "	INNER JOIN ".$table_prefix."_crmentity ON ".$table_prefix."_crmentity.crmid = $this->table_name.$this->table_index";

		// Consider custom table join as well.
		if(isset($this->customFieldTable)) {
			$from_clause .= " INNER JOIN ".$this->customFieldTable[0]." ON ".$this->customFieldTable[0].'.'.$this->customFieldTable[1] .
				      " = $this->table_name.$this->table_index"; 
		}
		$from_clause .= " LEFT JOIN ".$table_prefix."_users ON ".$table_prefix."_users.id = ".$table_prefix."_crmentity.smownerid
						LEFT JOIN ".$table_prefix."_groups ON ".$table_prefix."_groups.groupid = ".$table_prefix."_crmentity.smownerid";
		
		$where_clause = "	WHERE ".$table_prefix."_crmentity.deleted = 0";
		$where_clause .= $this->getListViewSecurityParameter($module);
					
		if (isset($select_cols) && trim($select_cols) != '') {
			$sub_query = "SELECT $select_cols FROM  $this->table_name AS t " .
				" INNER JOIN ".$table_prefix."_crmentity AS crm ON crm.crmid = t.".$this->table_index;
			// Consider custom table join as well.
			if(isset($this->customFieldTable)) {
				$sub_query .= " INNER JOIN ".$this->customFieldTable[0]." tcf ON tcf.".$this->customFieldTable[1]." = t.$this->table_index";
			}
			$sub_query .= " WHERE crm.deleted=0 GROUP BY $select_cols HAVING COUNT(*)>1";	
		} else {
			$sub_query = "SELECT $table_cols $from_clause $where_clause GROUP BY $table_cols HAVING COUNT(*)>1";
		}

		$query = $select_clause . $from_clause .
					" LEFT JOIN ".$table_prefix."_users_last_import ON ".$table_prefix."_users_last_import.bean_id=" . $this->table_name .".".$this->table_index .
					" INNER JOIN (" . $sub_query . ") temp ON ".get_on_clause($field_values,$ui_type_arr,$module) .
					$where_clause .
					" ORDER BY $table_cols,". $this->table_name .".".$this->table_index ." ASC";

		return $query;
	}

	// crmv@43050
	function save_related_module($module, $crmid, $with_module, $with_crmid) {
		global $adb, $table_prefix, $current_user;

		if ($module != 'ModComments') {
			return parent::save_related_module($module, $crmid, $with_module, $with_crmid);
		}

		if(!is_array($with_crmid)) $with_crmid = array($with_crmid);
		$with_crmid = array_filter($with_crmid);

		if ($crmid > 0 && $with_module && count($with_crmid) > 0)

		// save only the first
		$adb->pquery("update {$this->table_name} set related_to = ? where {$this->table_index} = ?", array($with_crmid[0], $crmid));
		$adb->pquery("update {$table_prefix}_crmentity set modifiedtime = ? where crmid = ?", array(date('Y-m-d H:i:s'), $crmid));

		if (isModuleInstalled('ModNotifications') && $current_user->id != $this->column_fields['assigned_user_id']) {
			$obj = CRMEntity::getInstance('ModNotifications');
			$owner = getOwnerId($crmid);
			$notified_users = $obj->saveFastNotification(
				array(
					'assigned_user_id' => $owner,
					'related_to' => $crmid,
					'mod_not_type' => 'Relation',
					'description' => $with_crmid[0],
				),false
			);

		}
	}
	// crmv@43050e

	/**
	 * Invoked when special actions are performed on the module.
	 * @param String Module name
	 * @param String Event Type (module.postinstall, module.disabled, module.enabled, module.preuninstall)
	 */
	function vtlib_handler($modulename, $event_type) {
		if($event_type == 'module.postinstall') {
			// TODO Handle post installation actions
		} else if($event_type == 'module.disabled') {
			// TODO Handle actions when this module is disabled.
		} else if($event_type == 'module.enabled') {
			// TODO Handle actions when this module is enabled.
		} else if($event_type == 'module.preuninstall') {
			// TODO Handle actions when this module is about to be deleted.
		} else if($event_type == 'module.preupdate') {
			// TODO Handle actions before this module is updated.
		} else if($event_type == 'module.postupdate') {
			// TODO Handle actions after this module is updated.
		}
	}

	/** 
	 * Handle saving related module information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	// function save_related_module($module, $crmid, $with_module, $with_crmid) { }
	
	/**
	 * Handle deleting related module information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	function delete_related_module($module, $crmid, $with_module, $with_crmid) {
		if ($with_module == 'ModComments') {
			$destinationModule = vtlib_purify($_REQUEST['destination_module']);
			if (!is_array($with_crmid)) $with_crmid = Array($with_crmid);
			foreach($with_crmid as $relcrmid) {
				$child = CRMEntity::getInstance($destinationModule);
				$child->retrieve_entity_info($relcrmid, $destinationModule);
				$child->trash($destinationModule, $relcrmid);
			}
		} else {
			parent::delete_related_module($module, $crmid, $with_module, $with_crmid);
		}
	}

	/**
	 * Handle getting related list information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	//function get_related_list($id, $cur_tab_id, $rel_tab_id, $actions=false) { }

	/**
	 * Handle getting dependents list information.
	 * NOTE: This function has been added to CRMEntity (base class).
	 * You can override the behavior by re-defining it here.
	 */
	//function get_dependents_list($id, $cur_tab_id, $rel_tab_id, $actions=false) { }
	
	function trash($module, $id) {
		global $current_user, $adb, $table_prefix;
		if ($this->column_fields['creator']) {
			$creator = $this->column_fields['creator'];
		} else {
			$focus = CRMEntity::getInstance($module);
			$focus->retrieve_entity_info($id, $module);
			$creator = $focus->column_fields['creator'];
		}
		if ($current_user->id != $creator) {
			return false;
		}
		require_once('modules/SDK/src/Notifications/Notifications.php');
		$notifications = new Notifications($current_user->id,'ModComments');
		$notifications->deleteAllNotificationForComment($id);
					
		parent::trash($module, $id);
		
		//cancello le eventuali risposte
		$result = $adb->pquery("SELECT modcommentsid FROM {$table_prefix}_modcomments WHERE parent_comments = ?",array($id));
		if ($result && $adb->num_rows($result) > 0) {
			while($row=$adb->fetchByAssoc($result)){ 
				$notifications->deleteAllNotificationForComment($row['modcommentsid']);
				parent::trash($module, $row['modcommentsid']);
			}
		}
	}
	
	function get_replies($id, $cur_tab_id, $rel_tab_id, $actions=false) {
		
		global $currentModule, $app_strings, $singlepane_view, $current_user,$table_prefix;

		$parenttab = getParentTab();
		
//		echo "$id, $currentModule";die;
		$focus = CRMEntity::getInstance($currentModule);
		$focus->retrieve_entity_info($id, $currentModule);
//	echo '<pre>';print_r($focus->column_fields);die;

		$related_module = vtlib_getModuleNameById($rel_tab_id);
		$other = CRMEntity::getInstance($related_module);
		
		// Some standard module class doesn't have required variables
		// that are used in the query, they are defined in this generic API
		vtlib_setup_modulevars($currentModule, $this);
		vtlib_setup_modulevars($related_module, $other);

		$button = '';

		// To make the edit or del link actions to return back to same view.
		if($singlepane_view == 'true') $returnset = "&return_module=$currentModule&return_action=DetailView&return_id=$id";
		else $returnset = "&return_module=$currentModule&return_action=CallRelatedList&return_id=$id";

		$return_value = null;
		$dependentFieldSql = $this->db->pquery("SELECT tabid, fieldname, columnname FROM ".$table_prefix."_field WHERE uitype='10' AND" .
				" fieldid IN (SELECT fieldid FROM ".$table_prefix."_fieldmodulerel WHERE relmodule=? AND module=?)", array($currentModule, $related_module));
		$numOfFields = $this->db->num_rows($dependentFieldSql);
		
		if($numOfFields > 0) {		
			$dependentColumn = $this->db->query_result($dependentFieldSql, 0, 'columnname');
			$dependentField = $this->db->query_result($dependentFieldSql, 0, 'fieldname');			
		
			$button .= '<input type="hidden" name="'.$dependentColumn.'" id="'.$dependentColumn.'" value="'.$id.'">';			
			$button .= '<input type="hidden" name="'.$dependentColumn.'_type" id="'.$dependentColumn.'_type" value="'.$currentModule.'">';
			if($actions) {
				if(is_string($actions)) $actions = explode(',', strtoupper($actions));
				if(in_array('ADD', $actions) && isPermitted($related_module,1, '') == 'yes' 
						&& getFieldVisibilityPermission($related_module,$current_user->id,$dependentField) == '0') {
					if (intval($focus->column_fields['parent_comments']) == 0) {
						$button .= '<input type="hidden" name="related_to" id="related_to" value="'.$focus->column_fields['related_to'].'">';
						$button .= "<input title='".getTranslatedString('LBL_ADD_NEW'). " ". getTranslatedString('LBL_MODCOMMENTS_REPLY',$related_module) ."' class='crmbutton small create'" .
							" onclick='this.form.action.value=\"EditView\";this.form.module.value=\"$related_module\"' type='submit' name='button'" .
							" value='". getTranslatedString('LBL_ADD_NEW'). " " . getTranslatedString('LBL_MODCOMMENTS_REPLY',$related_module) ."'>&nbsp;";
					}
				}
			}
			
			$query = "SELECT ".$table_prefix."_crmentity.*, $other->table_name.*";
	
			$query .= ", CASE WHEN (".$table_prefix."_users.user_name is not null) THEN ".$table_prefix."_users.user_name ELSE ".$table_prefix."_groups.groupname END AS user_name";
			
			$more_relation = '';
			if(!empty($other->related_tables)) {
				foreach($other->related_tables as $tname=>$relmap) {
					$query .= ", $tname.*";
	
					// Setup the default JOIN conditions if not specified
					if(empty($relmap[1])) $relmap[1] = $other->table_name;
					if(empty($relmap[2])) $relmap[2] = $relmap[0];
					$more_relation .= " LEFT JOIN $tname ON $tname.$relmap[0] = $relmap[1].$relmap[2]";
				}
			}
	
			$query .= " FROM $other->table_name";
			$query .= " INNER JOIN ".$table_prefix."_crmentity ON ".$table_prefix."_crmentity.crmid = $other->table_name.$other->table_index";
			//crmv@24527
			if (!empty($other->customFieldTable)) {
				$query .= " INNER JOIN ".$other->customFieldTable[0]." ON $other->table_name.$other->table_index = ".$other->customFieldTable[0].".".$other->customFieldTable[1];
			}
			//crmv@24527e
			$query .= " INNER  JOIN $this->table_name AS parent_modcomments ON parent_modcomments.$this->table_index = $other->table_name.$dependentColumn";
			$query .= $more_relation;
			$query .= " LEFT  JOIN ".$table_prefix."_users        ON ".$table_prefix."_users.id = ".$table_prefix."_crmentity.smownerid";
			if (!empty($other->groupTable) ){
				$query .= "	LEFT JOIN ".$other->groupTable[0]."
					ON ".$other->groupTable[0].".".$other->groupTable[1]." = $other->table_name.$other->table_index ";
				$query .= "	LEFT JOIN ".$table_prefix."_groups
					ON ".$other->groupTable[0].".groupname = ".$table_prefix."_groups.groupname ";
			}
			else {
				$query .= " LEFT  JOIN ".$table_prefix."_groups       ON ".$table_prefix."_groups.groupid = ".$table_prefix."_crmentity.smownerid";				
			}
			$query .= " WHERE ".$table_prefix."_crmentity.deleted = 0 AND parent_modcomments.$this->table_index = $id";
			$return_value = GetRelatedList($currentModule, $related_module, $other, $query, $button, $returnset);
		}
		if($return_value == null) $return_value = Array();
		$return_value['CUSTOM_BUTTON'] = $button;
		
		return $return_value;
	}
}
?>
