<?php
/*+**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 ************************************************************************************/
//crmv@start
require_once('modules/ModComments/models/Replies.php');

class ModComments_CommentsModel {
	
	protected $data;
	public $replies;
	protected $searchkey; //crmv@31301
	static $ownerNamesCache = array();
	static $ownerPhotoCache = array();
	
	function __construct($datarow, $searchkey='', $replyRows='') { //crmv@31301
		$this->data = $datarow;
		$this->searchkey = $searchkey;	//crmv@31301
		$replies = new ModComments_RepliesModel($this->id(), $searchkey, $replyRows); //crmv@31301
		$this->replies = $replies->getReplies();
	}
	
	function author() {
		$authorid = $this->data['smcreatorid'];
		if(!isset(self::$ownerNamesCache[$authorid])) {
			self::$ownerNamesCache[$authorid] = getUserFullName($authorid);
		}
		//crmv@31301
		if($this->searchkey && stristr(self::$ownerNamesCache[$authorid], $this->searchkey))
			return	'<mark>'.self::$ownerNamesCache[$authorid].'</mark>';
		else
		//crmv@31301e
			return self::$ownerNamesCache[$authorid];
	}
	
	// crmv@31780
	function recipients($nohtml = false) {
		$return = '';
		if ($this->data['visibility_comm'] == 'Users') {
			global $adb,$table_prefix;
			$result = $adb->pquery('SELECT * FROM '.$table_prefix.'_modcomments_users WHERE id = ?',array($this->id()));
			if ($result && $adb->num_rows($result) > 0) {
				$recipients = array();
				while($row=$adb->fetchByAssoc($result)) {
					if(!isset(self::$ownerNamesCache[$row['user']])) {
						self::$ownerNamesCache[$row['user']] = getUserFullName($row['user']);
					}
					//crmv@31301
					if($this->searchkey && stristr(self::$ownerNamesCache[$row['user']], $this->searchkey))
						$recipients[$row['user']] = '<mark>'.self::$ownerNamesCache[$row['user']].'</mark>';
					else
					//crmv@31301e
						$recipients[$row['user']] = self::$ownerNamesCache[$row['user']];
				}
				if (!empty($recipients)) {
					if (!$nohtml) $return .= ' > ';
					$info = array();
					foreach($recipients as $id => $name) {
						if(!isset(self::$ownerPhotoCache[$id])) {
							self::$ownerPhotoCache[$id] = getUserAvatar($id);
						}
						$info[] = array('value'=>$id,'name'=>$name,'img'=>self::$ownerPhotoCache[$id]);
					}
					if ($nohtml) {
						$return = $info;
					} else {
						$info = rawurlencode(Zend_Json::encode($info));
						if (count($recipients) > 1) {
							$return .= '<b><a href="javascript:;" style="text-decoration:none;" onmouseover="displayRecipientsInfo(this,\''.$info.'\');" title="'.implode(', ',$recipients).'">'.count($recipients).' '.getTranslatedString('LBL_USERS','Users').'</a></b>';
						} else {
							$return .= '<b><a href="javascript:;" style="text-decoration:none;" onmouseover="displayRecipientsInfo(this,\''.$info.'\');">'.implode('',$recipients).'</a></b>';
						}
					}
				}
			}
		}
		return $return;
	}
	// crmv@31780e
	
	function authorPhoto() {
		global $theme;
		$authorid = $this->data['smcreatorid'];
		if(!isset(self::$ownerPhotoCache[$authorid])) {
			self::$ownerPhotoCache[$authorid] = getUserAvatar($authorid);
		}
		return self::$ownerPhotoCache[$authorid];
	}
	
	function timestamp(){
		return getTranslatedString('LBL_DAY'.date('w',strtotime($this->data['modifiedtime'])),'Calendar').' '.getDisplayDate($this->data['modifiedtime']);
	}
	
	function timestampAgo(){
		if (in_array($this->data['modifiedtime'],array('','1970-01-01'))) {
			return '';
		}
		$difference = time() - strtotime($this->data['modifiedtime']);
		$periods = array("second", "minute", "hour", "day", "week", "month", "year", "decade");
		$lengths = array("60", "60", "24", "7", "4.35", "12", "10");
		for($j = 0; isset($lengths[$j]) && $difference >= $lengths[$j]; $j++) {
			$difference /= $lengths[$j];
		}
		$difference = round($difference);
		if($difference != 1) {
			$periods[$j].= "s";
		}
		$text = $difference.' '.getTranslatedString('lbl_'.$periods[$j],'ModComments').' '.getTranslatedString('LBL_AGO','ModComments');
		if ($difference == 0 &&  $periods[$j] == 'seconds') {
			$text = getTranslatedString('lbl_now','ModComments');
		}
		return $text;
	}
	
	function content() {
		//crmv@31301	//crmv@34684
		global $adb;
		$comments = str_ireplace($this->searchkey, '<mark>'.$this->searchkey.'</mark>', $this->data['commentcontent']);
		$comments = decode_html($comments);
		
		preg_match_all("/([\w]+?:\/\/.*?[^ \"\n\r\t<]*)/",$comments,$links1);
		preg_match_all("/((www|ftp)\.[\w\-]+\.[\w\-.\~]+(?:\/[^ \"\t\n\r<]*)?)/",$comments,$links2);
		$links = array_merge($links1,$links2);
		if (is_array($links)) {
			$links = $adb->flatten_array(array_filter($links));
			if (is_array($links)) {
				$links = array_filter($links,create_function('$var','if ($var == "" || $var == "www") return false; else return true;'));
				if (is_array($links)) {
					$links = array_unique($links);
				}
			}
		}
		
		$comments = preg_replace("/(^|[\n ])([\w]+?:\/\/.*?[^ \"\n\r\t<]*)/","\\1<a href=\"\\2\" target=\"_blank\">\\2</a>",$comments);
		$comments = preg_replace("/(^|[\n ])((www|ftp)\.[\w\-]+\.[\w\-.\~]+(?:\/[^ \"\t\n\r<]*)?)/","\\1<a href=\"http://\\2\" target=\"_blank\">\\2</a>",$comments);
		$comments = preg_replace("/(^|[\n ])([a-z0-9&\-_.]+?)@([\w\-]+\.([\w\-\.]+\.)*[\w]+)/i","\\1<a href=\"javascript:parent.jQuery('#ModCommentsNews').hide();parent.InternalMailer('\\2@\\3','','','','email_addy');\">\\2@\\3</a>",$comments);
		$comments = preg_replace("/,\"|\.\"|\)\"|\)\.\"|\.\)\"/","\"",$comments);
		$comments = nl2br($comments);
		
		if (!empty($links)) {
			foreach ($links as $link) {
				if (strlen($link) > 60) {
					$comments = str_replace(">$link<",'>'.substr($link,0,45).'...'.substr($link,-12).'<',$comments);
				}
			}
		}
		return $comments;
		//crmv@31301e	//crmv@34684e
	}

	// crmv@31780 - restituisce un array, no html
	function content_no_html() {

		// tolgo chiavi numeriche
		$cleandata = array();
		if (is_array($this->data) && count($this->data) > 0) {
			foreach ($this->data as $k=>$v) {
				if (!is_numeric($k)) $cleandata[$k] = $v;
			}
		}

		// author and recipients
		$cleandata['author'] = $this->author();
		$cleandata['authorPhoto'] = $this->authorPhoto();
		$cleandata['recipients'] = $this->recipients(true);

		// timestamp
		$cleandata['timestamp'] = $this->timestamp();
		$cleandata['timestampago'] = $this->timestampAgo();

		$cleandata['unseen'] = $this->isUnseen();
		// crmv@33311
		$relatedid = $this->relatedTo();
		if ($relatedid > 0) {
			$cleandata['related_to_module'] = getSalesEntityType($relatedid);
			$ename = getEntityName($cleandata['related_to_module'], array($relatedid));
			$cleandata['related_to_name'] = $ename[$relatedid];
		}
		// crmv@33311e

		// prendo le risposte
		if (is_array($this->replies) && count($this->replies) > 0) {
			$cleandata['replies'] = array();
			foreach ($this->replies as $rep) {
				$cleandata['replies'][] = $rep->content_no_html();
			}
		}
		return $cleandata;
	}
	// crmv@31780e
	
	function id() {
		return $this->data['crmid'];
	}
	
	function relatedTo() {
		return $this->data['related_to'];
	}
	
	function relatedToString() {
		$parent_id = $this->relatedTo();
		if (!in_array($parent_id,array('',0))) {
			$parent_module = getSalesEntityType($parent_id);			
	
			$entityType = getSingleModuleName($parent_module);
	
			$displayValueArray = getEntityName($parent_module, $parent_id);
			if(!empty($displayValueArray)){
				foreach($displayValueArray as $key=>$value){
					$displayValue = $value;
				}
			}
			return ' '.getTranslatedString('LBL_ABOUT','ModComments')." <a href='index.php?module=$parent_module&action=DetailView&record=$parent_id' title='$entityType' target='_parent'>$displayValue</a> ($entityType)";
		}
	}
	
	function isUnseen() {
		global $adb, $current_user;
		$result = $adb->pquery('SELECT * FROM vte_notifications where id = ? and userid = ? and type = ?',array($this->id(),$current_user->id,'ModComments'));
		if ($result && $adb->num_rows($result) > 0) {
			return true;
		}
		return false;
	}
	
	function authorId() {
		return $this->data['smcreatorid'];
	}
	
	function getCurrentUser() {
		global $current_user;
		return $current_user->id;
	}

	//crmv@43050

	// returns a list of all recipients of the message, including author and replies
	// useful only in case of users talk
	function getAllRecipients() {
		$rec = array($this->authorId());

		$replies = $this->recipients(true);
		if (is_array($replies)) {
			foreach ($replies as $r) $rec[] = $r['value'];
		}

		return array_unique($rec);
	}

	function isPublic() {
		return ($this->data['visibility_comm'] == 'All');
	}
	//crmv@43050e

}
//crmv@end
?>
