<?php
/*+**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 ************************************************************************************/
require_once('Smarty_setup.php');

class ModComments_DetailViewBlockCommentWidget {
	private $_name = 'DetailViewBlockCommentWidget';
	private $defaultCriteria = 'Last5';
	protected $context = false;
	protected $criteria= false;
	protected $defaultDisplayBlock = 'none';	// block/none
	protected $searchkey = ''; //crmv@31301

	function __construct() {
		//crmv@2963m
		if (($_REQUEST['module'] == 'Messages' || $_REQUEST['module'] == 'Emails') && $_REQUEST['file'] != 'PreView') {
			$this->defaultDisplayBlock = 'block;';
		}
		//crmv@2963me
	}

	function getFromContext($key, $purify=false) {
		if ($this->context) {
			$value = $this->context[$key];
			if ($purify && !empty($value)) {
				$value = vtlib_purify($value);
			}
			return $value;
		}
		return false;
	}

	function title() {
		return getTranslatedString('LBL_MODCOMMENTS_COMMUNICATIONS', 'ModComments');
	}

	function name() {
		return $this->_name;
	}

	function uikey() {
		return "ModCommentsDetailViewBlockCommentWidget";
	}

	function setCriteria($newCriteria) {
		$this->criteria = $newCriteria;
	}

	//crmv@31301
	function SetSearchKey($newSearchKey) {
		$this->searchkey = $newSearchKey;
	}
	//crmv@31301e

	function getViewer() {
		global $theme, $app_strings, $current_language;

		$smarty = new vtigerCRM_Smarty();
		$smarty->assign('APP', $app_strings);
		$smarty->assign('MOD', return_module_language($current_language,'ModComments'));
		$smarty->assign('THEME', $theme);
		$smarty->assign('IMAGE_PATH', "themes/$theme/images/");

		$smarty->assign('UIKEY', $this->uikey());
		$smarty->assign('WIDGET_TITLE', $this->title());
		$smarty->assign('WIDGET_NAME', $this->name());

		return $smarty;
	}

	protected function getModels($parentRecordId, $criteria) {
		global $adb, $current_user,$table_prefix;

		$moduleName = 'ModComments';
		if(vtlib_isModuleActive($moduleName)) {
			$entityInstance = CRMEntity::getInstance($moduleName);

			$queryCriteria  = '';
			switch($criteria) {
				case 'All':
				case 'Last5':
					$queryCriteria = '';
					break;
				case 'Mine':
					$queryCriteria = ' AND '.$table_prefix.'_crmentity.smownerid = '.$current_user->id;
					break;
			}
			$queryCriteria .= sprintf(" ORDER BY %s.%s DESC", $entityInstance->table_name, $entityInstance->table_index);
			$where = sprintf(" AND %s.related_to=?", $entityInstance->table_name);

			//$query = $entityInstance->getListQuery($moduleName, $where);
			$query = $entityInstance->getListQuery($moduleName, $where, false, true);
			$query .= $queryCriteria;
			//crmv@fix
			if ($criteria == 'Last5')
				$result = $adb->limitpQuery($query, 0, 5, array($parentRecordId));
			else
				//crmv@fix-e
				$result = $adb->pquery($query, array($parentRecordId));

			$instances = array();
			if($adb->num_rows($result)) {
				while($resultrow = $adb->fetch_array($result)) {
					$instances[] = new ModComments_CommentsModel($resultrow);
				}
			}
		}
		return $instances;
	}

	function getNewsModels($parentRecordId, $criteria, &$count) {
		global $adb, $current_user, $table_prefix;

		$moduleName = 'ModComments';
		if(vtlib_isModuleActive($moduleName)) {
			$entityInstance = CRMEntity::getInstance($moduleName);
			$query = $entityInstance->getListQuery($moduleName, '', false, true,true);	//crmv@32429 //crmv@36796
			$result = $adb->pquery($query, array($parentRecordId));
			$instances = array();
			if($adb->num_rows($result)) {
				while($resultrow = $adb->fetch_array($result)) {
					$instances[$resultrow['modcommentsid']] = $resultrow;
				}
			}
			$tmp_query = replaceSelectQuery($result->sql,$entityInstance->table_name.'.'.$entityInstance->table_index.'  AS id');
			$tableName = 'tmp_modcomments_u_'.$current_user->id;
			if ($adb->isMysql()) {
				$query = "create temporary table IF NOT EXISTS $tableName(id int(11) primary key) ignore ".$tmp_query;
				$result = $adb->query($query);
			} else {
				if (!$adb->table_exist($tableName,true)){
					Vtiger_Utils::CreateTable($tableName,"id I(11) NOTNULL PRIMARY",true,true);
				}
				$tableName = $adb->datadict->changeTableName($tableName);
				$query = "insert into $tableName select id from ($tmp_query) un_table where not exists (select * from $tableName where $tableName.id = un_table.id)";
				$result = $adb->query($query);
			}

			$where = '';
			$other_joins = '';
			if (!empty($this->searchkey)) {
				$where = " AND (commentcontent like '%".$this->searchkey."%'
								OR modcommUser.last_name like '%".$this->searchkey."%' OR modcommUser.first_name like '%".$this->searchkey."%'
								OR recipientUser.last_name like '%".$this->searchkey."%' OR recipientUser.first_name like '%".$this->searchkey."%'
								)";
				$column = 'user';
				$adb->format_columns($column);
				$other_joins = "INNER JOIN {$table_prefix}_users modcommUser ON crmentityReplies.smcreatorid = modcommUser.id
								LEFT JOIN {$table_prefix}_modcomments_users ON {$table_prefix}_modcomments_users.id = modcommReplies.modcommentsid
  								LEFT JOIN {$table_prefix}_users recipientUser ON {$table_prefix}_modcomments_users.$column = recipientUser.id";
			}
			// crmv@43448	crmv@36796
			if ($adb->isMssql()) {
				$unreadOrder = 'CASE WHEN notif.type IS NULL THEN 1 ELSE 0 END';
			} elseif ($adb->isOracle()) {
				$unreadOrder = 'notif.type DESC NULLS LAST';
			} else {
				$unreadOrder = 'notif.type DESC';
			}
			$query = "SELECT CASE WHEN (modcommReplies.parent_comments IS NULL OR modcommReplies.parent_comments = '') THEN modcommReplies.modcommentsid ELSE modcommReplies.parent_comments END AS id
						FROM {$table_prefix}_modcomments modcommReplies
						LEFT JOIN $tableName ON $tableName.id = modcommReplies.modcommentsid
						LEFT JOIN vte_notifications notif ON notif.id = modcommReplies.modcommentsid AND notif.userid = {$current_user->id}
						INNER JOIN {$table_prefix}_crmentity crmentityReplies ON crmentityReplies.crmid = modcommReplies.modcommentsid
						$other_joins
						WHERE modcommReplies.modcommentsid > 0
						    AND crmentityReplies.deleted = 0 $where
						ORDER BY $unreadOrder, crmentityReplies.modifiedtime DESC";
			$query = "select distinct t.id from ($query) t";
			$result = $adb->pquery($query, array($parentRecordId));
			
			$desc_ordered_modcommentsid = array();
			if ($result && $adb->num_rows($result) > 0) {
				while($row=$adb->fetchByAssoc($result)) {
					//if (!array_key_exists($row['id'],$desc_ordered_modcommentsid)) {
						if (isset($instances[$row['id']])) {
							$desc_ordered_modcommentsid[$row['id']] = $instances[$row['id']];
						}
					//}
				}
			}
			// crmv@43448e	crmv@36796e
			$count = @count($desc_ordered_modcommentsid);
			if ($count == 0) return array();
			
			$start = 0;
			// criteria can be: LastXXNews or PageYYNews
			if (preg_match('/Last(\d+)News/', $criteria, $matches)) {
				$length = empty($matches[1]) ? 20 : intval($matches[1]);
			} elseif (preg_match('/Page(\d+)News/', $criteria, $matches)) {
				$page = empty($matches[1]) ? 1 : intval($matches[1]);
				$length = 20;
				$start = ($page-1)*$length;
			} else {
				$length = 20;
			}

			$tmp_arr = array_slice($desc_ordered_modcommentsid,$start,$length);
			$tmp_ids = array();
			foreach ($tmp_arr as $info) {
				$tmp_ids[] = $info['modcommentsid'];
			}
			$query = $entityInstance->getListQuery('ModComments', " AND $entityInstance->table_name.parent_comments IN (".generateQuestionMarks($tmp_ids).")", true, true);
			$query .= sprintf(" ORDER BY %s.%s", $entityInstance->table_name, $entityInstance->table_index);
			$result = $adb->pquery($query, array($tmp_ids));
			$replies = array();
			if ($result && $adb->num_rows($result)) {
				while($row=$adb->fetchByAssoc($result)) {
					$replies[$row['parent_comments']][] = $row;
				}
			}
			foreach ($tmp_arr as $key => $info) {
				$tmp_arr[$key] = new ModComments_CommentsModel($info, $this->searchkey, $replies);
			}
			return $tmp_arr;
		}
	}

	// crmv@31780 -- get data as php array
	public function getModelsAsArray($context = false, &$count = null) {
		$this->context = $context;
		$sourceRecordId =  $this->getFromContext('ID', true);
		$usecriteria = ($this->criteria === false)? $this->defaultCriteria : $this->criteria;

		if (strpos($usecriteria,'News') !== false) {
			$models = $this->getNewsModels($sourceRecordId, $usecriteria, $count);
		} else {
			$models = $this->getModels($sourceRecordId, $usecriteria);
			$count = count($models);
		}

		$out = array();
		foreach ($models as $mod) {
			$row = $mod->content_no_html();
			$out[] = $row;
		}

		return $out;
	}
	// crmv@31780e

	function processItem($model) {
		$viewer = $this->getViewer();
		$viewer->assign('NEWS_MODE', 'yes'); // crmv@43050
		$viewer->assign('COMMENTMODEL', $model);
		$viewer->assign('DEFAULT_REPLY_TEXT', getTranslatedString('LBL_DEFAULT_REPLY_TEXT','ModComments'));
		return $viewer->fetch(vtlib_getModuleTemplate("ModComments","widgets/DetailViewBlockCommentItem.tpl"));
	}

	function process($context = false) {
		global $current_user;

		$this->context = $context;
		$sourceRecordId =  $this->getFromContext('ID', true);
		$usecriteria = ($this->criteria === false)? $this->defaultCriteria : $this->criteria;

		$viewer = $this->getViewer();
		$viewer->assign('ID', $sourceRecordId);
		$viewer->assign('CRITERIA', $usecriteria);
		$viewer->assign('DEFAULT_DISPLAY_BLOCK', $this->defaultDisplayBlock);

		$allow_generic_talks = 'yes';

		if (strpos($usecriteria,'News') !== false) {
			$viewer->assign('NEWS_MODE', 'yes');
			$comments = $this->getNewsModels($sourceRecordId, $usecriteria, $count);
			$length = explode('News',$usecriteria);
			$length = explode('Last',$length[0]);
			$length = $length[1];
			$viewer->assign('MAX_NUM_OF_NEWS', $length);
			$viewer->assign('TOTAL_NUM_OF_NEWS', $count);

			if ($current_user->column_fields['allow_generic_talks'] != 1) {
				$allow_generic_talks = 'no';
			}
		} else {
			$comments = $this->getModels($sourceRecordId, $usecriteria);
		}

		$viewer->assign('ALLOW_GENERIC_TALKS', $allow_generic_talks);

		$viewer->assign('COMMENTS', $comments);
		$viewer->assign('UNSEEN_IDS', $this->getUnseenComments($comments));

		//crmv@35267
		if ($current_user->column_fields['receive_public_talks'] == 1) {
			$viewer->assign('ENABLE_PUBLIC_TALKS', true);
		}
		//crmv@35267e

		//crmv@2963m
		if ($_REQUEST['module'] == 'Messages' || $_REQUEST['module'] == 'Emails')
			return $viewer->fetch(vtlib_getModuleTemplate("ModComments","widgets/DetailViewBlockCommentMessages.tpl"));
		else
		//crmv@2963me
			return $viewer->fetch(vtlib_getModuleTemplate("ModComments","widgets/DetailViewBlockComment.tpl"));
	}

	function processItemReply($model) {
		$viewer = $this->getViewer();
		$viewer->assign('REPLYMODEL', $model);
		$viewer->assign('UIKEY', $this->uikey());
		return $viewer->fetch(vtlib_getModuleTemplate("ModComments","widgets/DetailViewBlockReplyItem.tpl"));
	}

	function getUnseenComments($comments) {
		$return = array();
		if (!empty($comments)) {
			foreach($comments as $comment) {
				if ($comment->isUnseen()) {
					$return[] = $comment->id();
				}
				if (!empty($comment->replies)) {
					foreach($comment->replies as $reply) {
						if ($reply->isUnseen()) {
							$return[] = $reply->id();
						}
					}
				}
			}
		}
		return $return;
	}
}
?>
