<?php
/* crmv@31780 */
/* crmv@33097 - offline */
/* crmv@34559 - commenti,filtri */
/* crmv@38476 - sdk */
/* crmv@39106 - documenti, pdfmaker */
/* crmv@39110 - profilazione */

/*
 * TODO Touch:
 * 1. gestione centralizzata dei messaggi di errore
 * 2. uniformare tutte le richieste ajax (ritornare sempre json)
 * 3. spostare tutte le funzioni di TouchUtils qui dentro
 */

require_once('modules/Touch/TouchWSClass.php');
require_once('modules/Touch/TouchUtils.php');

class Touch extends SDKExtendableUniqueClass { // crmv@42024

	// versione API Touch lato server - quando si cambia, l'app mostra un warning al login
	public $version = "1.5";

	// limite di elementi per pagina
	public $listPageLimit = 20;

	// moduli da escludere dall'app (totalmente)
	public $excluded_modules = array('Fax', 'Sms', 'Emails', 'ChangeLog', 'Charts', 'PriceBooks', 'PBXManager', 'WebMails', 'Projects', 'Newsletter', 'Targets', 'Campaigns', 'Events', 'Telemarketing', 'Transitions',); // crmv@33311
	// da escludere solo per la lista moduli
	public $excluded_modules_list = array('ModNotifications', 'ModComments');

	// numero massimo di record da scaricare per l'offline
	// vengono presi i più recenti
	public $offline_max_items = 100;
	// numero di record da scaricare alla volta
	public $offline_chunks = 50;

	// lista dei webservices
	public $webservices = array(
		'Login' => 'TouchLogin.php',
		'ModulesList' => 'TouchModulesList.php',
		'GetBlocks' => 'TouchGetBlocks.php',
		'GetComments' => 'TouchGetComments.php',
		'GetTicketComments' => 'TouchGetTicketComments.php',
		'WriteComment' => 'TouchWriteComment.php',
		'WriteTicketComment' => 'TouchWriteTicketComment.php',
		'GetFavorites' => 'TouchGetFavorites.php',
		'SetFavorites' => 'TouchSetFavorites.php',
		'GetNotifications' => 'TouchGetNotifications.php',
		'SeeNotifications' => 'TouchSeeNotifications.php',
		'GetRecents' => 'TouchGetRecents.php',
		'GetTodos' => 'TouchGetTodos.php',
		'GetList' => 'TouchGetList.php',
		'GetRecord' => 'TouchGetRecord.php',
		'GetRelated' => 'TouchGetRelated.php',
		'GetUsers' => 'TouchGetUsers.php',
		'GetGroups' => 'TouchGetGroups.php',
		'SaveRecord' => 'TouchSaveRecord.php',
		'DeleteRecord' => 'TouchDeleteRecord.php',
		'GetAssociatedProducts' => 'TouchGetAssociatedProducts.php',
		'DeleteRelation' => 'TouchDeleteRelation.php',
		'SaveRelation' => 'TouchSaveRelation.php',
		'SaveFilter' => 'TouchSaveFilter.php',
		'GetOfflineData' => 'TouchGetOfflineData.php',
		'GetOverrideFile' => 'TouchGetOverrideFile.php',
		'PDFMaker' => 'TouchPDFMaker.php',
		'SimpleEdit' => 'TouchSimpleEdit.php', // crmv@39110
		'GetMenuList' => 'TouchGetMenuList.php', // crmv@42707
		'GetMessagesMeta' => 'TouchGetMessagesMeta.php', // crmv@42537
		'GetMessagesCount' => 'TouchGetMessagesCount.php',
		'Autocomplete' => 'TouchAutocomplete.php',
		'SendMail' => 'TouchSendMail.php',
		'SetFlag' => 'TouchSetFlag.php',
		'GetLinkedRecords' => 'TouchGetLinkedRecords.php',
		'LinkModules' => 'TouchLinkModules.php',
	);

	// cache per le descrizioni dei campi
	protected $moduleFieldsCache = array();

	public function isOfflineEnabled() {
		return true;
	}

	// restituisce il nome del file per il webservice specificato
	public function getWSFile($wsname) {
		return 'modules/Touch/vtws/'.$this->webservices[$wsname];
	}

	public function getWSClassFile($wsname) {
		return 'modules/Touch/vtws/classes/'.str_replace('.', '', $wsname).'.class.php';
	}

	// trasforma i valori dei campi da VTE a APP
	// se onlydisplay = true, formatta il campo per la visualizzazione
	public function field2Touch($module, $fieldname, $fieldvalue, $onlydisplay = false, &$focus = null) {
		global $log, $adb, $table_prefix, $current_user, $site_URL;

		if (!array_key_exists($module, $this->moduleFieldsCache)) {
			$response = wsRequest($current_user->id,'describe',	array('elementType'=>$module));
			$fields = $response['result']['fields'];
			if (is_array($fields)) {
				$this->moduleFieldsCache[$module] = $fields;
			}
		}

		$k = array_usearch($this->moduleFieldsCache[$module], create_function('$f', 'return $f["name"] == "'.$fieldname.'";'));
		$fieldinfo = $this->moduleFieldsCache[$module][$k];
		if (is_array($fieldinfo)) {
			$type = $fieldinfo['type']['name'];
			switch ($type) {
				case 'reference':
					$fieldvalue = intval($fieldvalue);

					if ($fieldvalue > 0) {
						// trovo il modulo collegato (dato che possono essere multipli)
						$setype = getSalesEntityType($fieldvalue);
						if (!empty($setype) && in_array($setype, $fieldinfo['type']['refersTo'])) {

							$displayname = '';
							$wsfield = VtigerWebserviceObject::fromName($adb, $setype);
							if ($wsfield) {
								$handlerClass = $wsfield->getHandlerClass();
								$inst = new $handlerClass($wsfield, $current_user, $adb, $log);
								$wsrecordid = vtws_getWebserviceEntityId($setype, $fieldvalue);
								$modinfo = $inst->describe($setype); // TODO: poco efficiente!
								$data = $inst->retrieve($wsrecordid);
								$labels = explode(',', $modinfo['labelFields']);
								foreach ($labels as $labelname) {
									$displayname .= $data[$labelname]." ";
								}
							}
							$fieldvalue = array('crmid'=>$fieldvalue, 'display'=>$displayname, 'setype'=>$setype);
						}
					}
					if ($onlydisplay) {
						$fieldvalue = (empty($displayname) ? '' : $displayname);
					}
					break;
				case 'date':
					// reverse date format
					if (preg_match('/\d{2}-\d{2}\-\d{4}/', $fieldvalue)) {
						$ndate = date_parse_from_format('d-m-Y', $fieldvalue);
						$fieldvalue = $ndate['year'].'-'.$ndate['month'].'-'.$ndate['day'];
					}
					break;
				case 'picklist':
					if ($onlydisplay)
						$fieldvalue = getTranslatedString($fieldvalue, $module);
					break;
				case 'picklistmultilanguage':
					if ($onlydisplay) {
						$plid = $fieldvalue;
						$fieldvalue = Picklistmulti::getTranslatedPicklist($plid, $fieldname);
					}
					break;
				case 'multipicklist':
					if ($onlydisplay) {
						$values = explode(' |##| ', $fieldvalue);
						if (count($values) > 0) {
							foreach ($values as $k=>$v) $values[$k] = getTranslatedString($v, $module);
						}
						$fieldvalue = implode(', ', $values);
					} else {
						$fieldvalue = str_replace(' |##| ', ',', $fieldvalue);
					}
					break;
				case 'boolean':
					if ($onlydisplay) {
						$fieldvalue = getTranslatedString(($fieldvalue ? 'LBL_YES' : 'LBL_NO'), 'APP_STRINGS');
					}
					break;
				case 'owner':
					$username = getUserName($fieldvalue);
					if ($onlydisplay) $fieldvalue = $username;
					break;
				case 'file':
					// fix internal link
					if (!empty($fieldvalue) && $focus && $module == 'Documents' && $focus->column_fields['filelocationtype'] == 'I' && $focus->column_fields['filestatus'] == 1) {
						// get direct link
						$res = $adb->pquery("select * from {$table_prefix}_seattachmentsrel inner join {$table_prefix}_attachments on {$table_prefix}_attachments.attachmentsid = {$table_prefix}_seattachmentsrel.attachmentsid where {$table_prefix}_seattachmentsrel.crmid = ?", array($focus->column_fields['record_id']));
						if ($res && $adb->num_rows($res) > 0) {
							$attid = $adb->query_result_no_html($res, 0, 'attachmentsid');
							$name = $adb->query_result_no_html($res, 0, 'name');
							$filepath = $adb->query_result($res, 0, 'path');
							$saved_filename = $attid."_".$name;
							$fieldvalue = $site_URL."/".$filepath.$saved_filename;
							//$fieldvalue = $site_URL."/index.php?module=uploads&action=downloadfile&fileid={$attid}&entityid={$focus->column_fields['record_id']}";
						}
					}
					break;
			}
			// campi documenti
			if ($module == 'Documents' && $fieldname == 'filesize' && !empty($fieldvalue)) {
				if ($fieldvalue < 1024)
					$fieldvalue = $fieldvalue.' B';
				elseif ($fieldvalue > 1024 && $fieldvalue < 1048576)
					$fieldvalue = round($fieldvalue/1024,2).' KB';
				else
					$fieldvalue = round($fieldvalue/(1024*1024),2).' MB';
			}
			// campi cifrati
			if ($fieldinfo['uitype'] == 208) {
				$fieldvalue = "-- ".getTranslatedString('LBL_CIPHERED', 'APP_STRINGS')." --";
			}
		}

		return $fieldvalue;
	}

	// trasforma i valori dei campi da APP a VTE
	public function touch2Field($module, $fieldname, $fieldvalue) {
		global $current_user;

		if (!array_key_exists($module, $this->moduleFieldsCache)) {
			$response = wsRequest($current_user->id,'describe',	array('elementType'=>$module));
			$fields = $response['result']['fields'];
			if (is_array($fields)) {
				$this->moduleFieldsCache[$module] = $fields;
			}
		}

		$k = array_usearch($this->moduleFieldsCache[$module], create_function('$f', 'return $f["name"] == "'.$fieldname.'";'));
		$fieldinfo = $this->moduleFieldsCache[$module][$k];
		if (is_array($fieldinfo)) {
			$type = $fieldinfo['type']['name'];
			switch ($type) {
				case 'owner':
					$fieldvalue = intval($fieldvalue);
					if ($fieldvalue <= 0) $fieldvalue = 1; // assegno ad admin in caso di dato non valido
					// crmv@34947
					try {
						$otype = vtws_getOwnerType($fieldvalue);
					} catch (Exception $e) {
						$otype = 'Users';
					}
					$fieldvalue = vtws_getWebserviceEntityId($otype, $fieldvalue);
					// crmv@34947e
					break;
				case 'date':
					$fieldvalue = substr(str_replace('T', ' ', $fieldvalue), 0, 10);
					break;
				case 'datetime':
				case 'timestamp':
					$fieldvalue = str_replace('T', ' ', $fieldvalue);
					break;

				// gli id arrivano in formato ID, non webservice
				case 'reference': {
					$fieldvalue = intval($fieldvalue);

					if ($fieldvalue > 0) {
						// trovo il modulo collegato (dato che possono essere multipli)
						if ($fieldinfo['type']['refersTo'][0] == 'DocumentFolders') {
							$setype = 'DocumentFolders';
						} else {
							$setype = getSalesEntityType($fieldvalue);
							if (empty($setype)) $setype = $fieldinfo['type']['refersTo'][0]; // for Currency and maybe others
						}
						if (!empty($setype) && in_array($setype, $fieldinfo['type']['refersTo'])) {
							$fieldvalue = vtws_getWebserviceEntityId($setype, $fieldvalue);
						}
					} else {
						$fieldvalue = '';
					}
					break;
				}
				case 'multipicklist':
					// TODO: e se i valori contengono la virgola?
					$fieldvalue = str_replace(',', ' |##| ', $fieldvalue);
					break;
				case 'boolean':
					$fieldvalue = ($fieldvalue ? 'on' : 'off');
					break;
				default:
					$fieldvalue = str_replace('&amp;', '&', $fieldvalue);
					break;
			}
		}

		return $fieldvalue;
	}

	function vtlib_handler($modulename, $event_type) {
		global $adb, $table_prefix;

		if($event_type == 'module.postinstall') {
			$registerWSAPI = true;
			$adb->pquery("UPDATE {$table_prefix}_tab SET customized = '0' WHERE name=?", array($moduleName));
		} else if($event_type == 'module.disabled') {
			// TODO Handle actions when this module is disabled.
		} else if($event_type == 'module.enabled') {
			// TODO Handle actions when this module is enabled.
		} else if($event_type == 'module.preuninstall') {
			// TODO Handle actions when this module is about to be deleted.
		} else if($event_type == 'module.preupdate') {
			// TODO Handle actions before this module is updated.
		} else if($event_type == 'module.postupdate') {
		}
	}
}

?>
