<?php
/***************************************************************************************
 * The contents of this file are subject to the CRMVILLAGE.BIZ VTECRM License Agreement
 * ("licenza.txt"); You may not use this file except in compliance with the License
 * The Original Code is:  CRMVILLAGE.BIZ VTECRM
 * The Initial Developer of the Original Code is CRMVILLAGE.BIZ.
 * Portions created by CRMVILLAGE.BIZ are Copyright (C) CRMVILLAGE.BIZ.
 * All Rights Reserved.
 ***************************************************************************************/

/*
 * crmv@37004 - get relations between modules (beta version)
 */

// represent a single relation between 2 modules
class ModuleRelation {

	// relation type
	public static $TYPE_1TO1 = 1;	// TODO
	public static $TYPE_1TON = 2;
	public static $TYPE_NTO1 = 4;
	public static $TYPE_NTON = 8;

	public static $TYPE_ALL = 0x0F;

	// modules
	protected $type;
	protected $module1;
	protected $module2;

	// relation info 1-N, N-1
	public $fieldid;
	public $fieldname;
	public $fieldtable;
	public $fieldcolumn;

	// relation info N-N
	public $relationid;

	function __construct($module1, $module2, $type) {
		$this->module1 = $module1;
		$this->module2 = $module2;
		$this->type = $type;
	}

	function getFirstModule() {
		return $this->module1;
	}

	function getSecondModule() {
		return $this->module2;
	}

	function getType() {
		return $this->type;
	}

	function getFieldId() {
		return $this->fieldid;
	}
	//crmv@47905
	function getNtoNinfo(){
		global $table_prefix;
		$mod1Inst = CRMEntity::getInstance($this->module1);
		$mod2Inst = CRMEntity::getInstance($this->module2);
		// module1 -> module2
		$reltab = $mod1Inst->relation_table;
		$relidxCrmid = $mod1Inst->table_index;
		$relidx = $mod1Inst->relation_table_id;
		$relidx2 = $mod1Inst->relation_table_otherid;
		$relmod1 = $mod1Inst->relation_table_module;
		$relmod2 = $mod1Inst->relation_table_othermodule;
		//crmv@43864
		if ($this->module1 == 'Messages') {
			$relidxCrmid = 'messagehash';
		} elseif (isInventoryModule($this->module1) && isProductModule($this->module2)) {
			// inventorymod->products
			$reltab = $table_prefix."_inventoryproductrel";
			$relidx = 'id';
			$relidx2 = 'productid';
			$relmod1 = $relmod2 = '';
		} elseif (isProductModule($this->module1) && isInventoryModule($this->module2)) {
			// products->inventorymod
			$reltab = $table_prefix."_inventoryproductrel";
			$relidx = 'productid';
			$relidx2 = 'id';
			$relmod1 = $relmod2 = '';
		} elseif ($this->module1 == 'PriceBooks' && isProductModule($this->module2)) {
			$reltab = $table_prefix."_pricebookproductrel";
			$relidx = 'pricebookid';
			$relidx2 = 'productid';
			$relmod1 = $relmod2 = '';
		} elseif (isProductModule($this->module1) && $this->module2 == 'PriceBooks') {
			$reltab = $table_prefix."_pricebookproductrel";
			$relidx = 'productid';
			$relidx2 = 'pricebookid';
			$relmod1 = $relmod2 = '';
			// crmv@44187
			// TODO: use this system also for products/pricebooks/inventory
		} elseif ($this->module2 == 'Products' || $this->module2 == 'Documents'){
			$reltab = $mod2Inst->relation_table;
			$relidx = $mod2Inst->relation_table_otherid;
			$relidx2 = $mod2Inst->relation_table_id;
			$relmod1 = $relmod2 = '';
		} else if (!empty($mod1Inst->extra_relation_tables[$this->module2])) {
			$reltab = $mod1Inst->extra_relation_tables[$this->module2]['relation_table'];
			$relidx = $mod1Inst->extra_relation_tables[$this->module2]['relation_table_id'];
			$relidx2 = $mod1Inst->extra_relation_tables[$this->module2]['relation_table_otherid'];
			$relmod1 = $mod1Inst->extra_relation_tables[$this->module2]['relation_table_module'];
			$relmod2 = $mod1Inst->extra_relation_tables[$this->module2]['relation_table_othermodule'];
		}
		// crmv@44187e
		return Array(
			'reltab'=>$reltab,
			'relidxCrmid'=>$relidxCrmid,
			'relidx'=>$relidx,
			'relidx2'=>$relidx2,
			'relmod1'=>$relmod1,
			'relmod2'=>$relmod2,
		);
	}
	//crmv@47905 e
	// returns related ids for this relation, starting with specified module
	// TODO: i permessi!!
	// TODO: fix limit for N-N relations
	// crmv@43611
	function getRelatedIds($crmid, $start = 0, $limit = 0, $onlycount = false) {
		global $adb, $table_prefix;

		$ret = array();
		$count = 0;

		switch ($this->type) {
			case self::$TYPE_1TON: {
				// fieldname is in secondary module
				$r = self::searchFieldValue($this->fieldid, $crmid, $start, $limit);
				if ($r) {
					$ret = $r;
					++$count;
				}
				break;
			}
			case self::$TYPE_NTO1: {
				// fieldname is in primary module
				if (!empty($this->fieldid)) {
					$v = getFieldValue($this->fieldid, $crmid);
					if ($v && getSalesEntityType($v) == $this->module2) { // crmv@44187
						$ret[] = $v;
						++$count;
					}
				}
				break;
			}
			case self::$TYPE_NTON: {
				$ntoninfo = $this->getNtoNinfo(); //crmv@47905
				$mod1Inst = CRMEntity::getInstance($this->module1);
				$mod2Inst = CRMEntity::getInstance($this->module2);
				$reltab = $ntoninfo['reltab'];
				$relidxCrmid = $ntoninfo['relidxCrmid'];
				$relidx = $ntoninfo['relidx'];
				$relidx2 = $ntoninfo['relidx2'];
				$relmod1 = $ntoninfo['relmod1'];
				$relmod2 = $ntoninfo['relmod2'];
				//crmv@43864e
				$query =
					"select ".($onlycount ? 'count(crm2.crmid) as cnt' : 'crm2.crmid')."
					from $reltab r
						inner join {$mod1Inst->table_name} mtab on mtab.$relidxCrmid = r.$relidx
						inner join {$table_prefix}_crmentity crm on crm.crmid = mtab.{$mod1Inst->table_index}
						inner join {$table_prefix}_crmentity crm2 on crm2.crmid = r.$relidx2
					where crm.deleted = 0 and crm2.deleted = 0 and mtab.{$mod1Inst->table_index} = ? and crm2.setype = ?";
				$params = array($crmid, $this->module2);
				if ($relmod1) {
					$query .= " and r.$relmod1 = ?";
					$params[] = $this->module1;
				}
				if ($relmod2) {
					$query .= " and r.$relmod2 = ?";
					$params[] = $this->module2;
				}

				if ($reltab && $relidxCrmid && $relidx && $relidx2) {
					if ($limit > 0 && !$onlycount) {
						$res = $adb->limitpQuery($query, $start, $limit, $params);
					} else {
						$res = $adb->pquery($query, $params);
					}
					if ($res && $adb->num_rows($res) > 0) {
						if ($onlycount) {
							$count += intval($adb->query_result_no_html($res, 0, 'cnt'));
						} else {
							while ($row = $adb->FetchByAssoc($res, -1, false)) {
								$ret[] = $row['crmid'];
							}
						}
					}
				}

				// module2 -> module1
				$reltab = $mod2Inst->relation_table;
				$relidxCrmid = $mod2Inst->table_index;
				$relidx = $mod2Inst->relation_table_id;
				$relidx2 = $mod2Inst->relation_table_otherid;
				$relmod1 = $mod2Inst->relation_table_module;
				$relmod2 = $mod2Inst->relation_table_othermodule;
				if ($this->module2 == 'Messages') {
					$relidxCrmid = 'messagehash';
				}
				$query =
					"select ".($onlycount ? 'count(crm.crmid) as cnt' : 'crm.crmid')."
					from $reltab r
						inner join {$mod2Inst->table_name} mtab on mtab.$relidxCrmid = r.$relidx
						inner join {$table_prefix}_crmentity crm on crm.crmid = mtab.{$mod2Inst->table_index}
						inner join {$table_prefix}_crmentity crm2 on crm2.crmid = r.$relidx2
					where crm.deleted = 0 and crm2.deleted = 0 and crm2.crmid = ? and crm.setype = ?";
				$params = array($crmid, $this->module2);
				if ($relmod1) {
					$query .= " and r.$relmod1 = ?";
					$params[] = $this->module2;
				}
				if ($relmod2) {
					$query .= " and r.$relmod2 = ?";
					$params[] = $this->module1;
				}

				if ($reltab && $relidxCrmid && $relidx && $relidx2) {
					if ($limit > 0 && !$onlycount) {
						$res = $adb->limitpQuery($query, $start, $limit, $params);
					} else {
						$res = $adb->pquery($query, $params);
					}
					if ($res && $adb->num_rows($res) > 0) {
						if ($onlycount) {
							$count += intval($adb->query_result_no_html($res, 0, 'cnt'));
						} else {
							while ($row = $adb->FetchByAssoc($res, -1, false)) {
								$ret[] = $row['crmid'];
							}
						}
					}
				}

				break;
			}
		}

		return ($onlycount ? $count : $ret);
	}

	function countRelatedIds($crmid) {
		return $this->getRelatedIds($crmid, 0,0, true);
	}
	// crmv@43611e

	// search for crmids which field value are $value
	// returns array of ids
	static function searchFieldValue($fieldid, $value, $start = 0, $limit = 0) {
		global $adb, $table_prefix;

		$query =
			"select
				{$table_prefix}_tab.name as modulename,	fieldid, fieldname, tablename, columnname
			from {$table_prefix}_field
				inner join {$table_prefix}_tab on {$table_prefix}_tab.tabid = {$table_prefix}_field.tabid
			where fieldid=? and {$table_prefix}_field.presence in (0,2)";

		if ($limit > 0) {
			$res = $adb->limitpQuery($query, $start, $limit, array($fieldid));
		} else {
			$res = $adb->pquery($query, array($fieldid));
		}

		if ($res && $adb->num_rows($res) > 0) {

			$row = $adb->FetchByAssoc($res, -1, false);
			$focus = CRMEntity::getInstance($row['modulename']);
			if (empty($focus)) return null;

			$indexname = $focus->tab_name_index[$row['tablename']];
			if (empty($indexname)) return null;

			$ret = array();
			if ($row['tablename'] != "{$table_prefix}_crmentity") {
				$join = "inner join {$table_prefix}_crmentity on {$table_prefix}_crmentity.crmid = {$row['tablename']}.$indexname";
			} else {
				$join = "";
			}
			// TODO: add converted = 0 for leads
			$res2 = $adb->pquery("select {$table_prefix}_crmentity.crmid from {$row['tablename']} $join where {$table_prefix}_crmentity.deleted = 0 and {$row['tablename']}.{$row['columnname']} = ? and {$table_prefix}_crmentity.setype = ?", array($value,$row['modulename'])); // crmv@43765
			if ($res2 && $adb->num_rows($res2) > 0) {
				while ($row2 = $adb->FetchByAssoc($res2, -1, false)) {
					$ret[] = $row2['crmid'];
				}
				return $ret;
			}
		}
		return null;
	}
}

class RelationManager extends SDKExtendableUniqueClass { // crmv@42024

	// user object to use when getting privileges
	protected $user = null;

	// special uitypes for relations (uitype->array(destination modules))
	protected $uitype_rel = array(
		'51' => array("Accounts"),
		'57' => array("Contacts"),
		'58' => array("Campaigns"),
		'59' => array("Products"),
		// Calendar related_to : dynamically populated
		'66' => array(),
		'68' => array('Accounts','Contacts'),
		'73' => array("Accounts"),
		'75' => array("Vendors"),
		'76' => array("Potentials"),
		'78' => array("Quotes"),
		'81' => array("Vendors"),
		'80' => array("SalesOrder"),
		'206' => array("Reports"),
		// TODO: users
	);

	protected $uitype_relid;
	protected $relations; // cache (by module and relation type)
	protected $disableCache = false;

	protected function __construct($user = null) {
		global $current_user;

		$this->user = (empty($user) ? $current_user : $user);
		$this->relations = array();

		// create dynamic uitype modules list
		$calmod = getCalendarRelatedToModules();
		if (count($calmod) > 0)  $this->uitype_rel['66'] = $calmod;

		// get the tabids for special  uitypes
		foreach ($this->uitype_rel as $uitype=>$modlist) {
			$list = array_map('getTabid', $modlist);
			$this->uitype_relid[$uitype] = $list;
		}

	}

	// ----- cache related functions -----
	function disableCache() {
		$this->disableCache = true;
		$this->clearCache();
	}

	function enableCache() {
		$this->disableCache = false;
	}

	function getCachedRelations($module, $type, $relmodules = array(), $excludeModules = array()) {
		$ret = $this->relations[$module][$type];
		// FILTER modules
		if (is_array($ret) && (count($relmodules) > 0 || count($excludeModules) > 0)) {
			foreach ($ret as $k=>$relation) {
				$destmod = $relation->getSecondModule();
				if (!in_array($destmod, $relmodules) || in_array($destmod, $excludeModules)) unset($ret[$k]);
			}
		}
		return $ret;
	}

	function hasCachedRelation($module, $type) {
		return (!$this->disableCache && is_array($this->relations[$module][$type]));
	}

	function initializeCache($module, $type) {
		$this->relations[$module][$type] = array();
	}

	function addRelationToCache($relation) {
		$module = $relation->getFirstModule();
		$type = $relation->getType();
		$this->relations[$module][$type][] = $relation;
	}

	function clearCache() {
		$this->relations = null;
	}
	// ----- end cache related functions -----

	// return a list of all modules related to the specified module (optionally filtered for destination module)
	// type is a OR between ModuleRelation::$TYPE_*
	function getRelations($module, $type = null, $relmodules = array(), $excludeModules = array()) {
		global $adb, $table_prefix;

		if (is_null($type)) $type = ModuleRelation::$TYPE_ALL;
		if (!is_array($relmodules)) $relmodules = array($relmodules);
		if (!is_array($excludeModules)) $excludeModules = array($excludeModules);

		$relmodules = array_diff($relmodules, $excludeModules);

		// save cache only if no module filtering
		$saveCache = (!$this->disableCache && (count($relmodules) == 0) && (count($excludeModules) == 0));

		$moduleid = getTabid($module);
		$moduleInst = CRMEntity::getInstance($module);

		$relations = array();

		// N-1
		if ($type & ModuleRelation::$TYPE_NTO1) {
			if ($this->hasCachedRelation($module, ModuleRelation::$TYPE_NTO1)) {
				$relations = array_merge($relations, $this->getCachedRelations($module, ModuleRelation::$TYPE_NTO1, $relmodules, $excludeModules));
			} else {
				if ($saveCache) $this->initializeCache($module, ModuleRelation::$TYPE_NTO1);
				$query = "select fmr.fieldid,fmr.relmodule, f.fieldname, f.tablename, f.columnname from {$table_prefix}_fieldmodulerel fmr inner join {$table_prefix}_field f on f.fieldid = fmr.fieldid where fmr.module = ?"; // crmv@42752
				$params = array($module);

				if (!empty($relmodules)) {
					$query .= " and fmr.relmodule in (".generateQuestionMarks($relmodules).")";
					$params[] = $relmodules;
				}
				if (!empty($excludeModules)) {
					$query .= " and fmr.relmodule not in (".generateQuestionMarks($excludeModules).")";
					$params[] = $excludeModules;
				}
				$res = $adb->pquery($query, $params);
				if ($res) {
					while ($row = $adb->FetchByAssoc($res, -1, false)) {
						$newrel = new ModuleRelation($module, $row['relmodule'], ModuleRelation::$TYPE_NTO1);
						$newrel->fieldid = $row['fieldid'];
						$newrel->fieldname = $row['fieldname'];
						$newrel->fieldtable = $row['tablename'];
						$newrel->fieldcolumn = $row['columnname'];
						$relations[] = $newrel;
						if ($saveCache) $this->addRelationToCache($newrel);
					}
				}

				// special uitypes, N-1
				$uitypeRelFiltered = $this->uitype_rel;
				if (!empty($relmodules)) {
					foreach ($uitypeRelFiltered as $ukey=>$kmods) {
						$kmods2 = array_intersect($kmods, $relmodules);
						if (count($kmods2) == 0) {
							unset($uitypeRelFiltered[$ukey]);
						} elseif (count($kmods2) != count($kmods)) {
							$uitypeRelFiltered[$ukey] = $kmods2;
						}
					}
				} elseif (!empty($excludeModules)) {
					foreach ($uitypeRelFiltered as $ukey=>$kmods) {
						$kmods2 = array_diff($kmods, $excludeModules);
						if (count($kmods2) == 0) {
							unset($uitypeRelFiltered[$ukey]);
						} elseif (count($kmods2) != count($kmods)) {
							$uitypeRelFiltered[$ukey] = $kmods2;
						}
					}
				}
				if (count($uitypeRelFiltered) > 0) {
					$uitypeFields = array_keys($uitypeRelFiltered);
					$res = $adb->pquery("select fld.fieldid, fld.uitype, fld.fieldname, fld.tablename, fld.columnname from {$table_prefix}_field fld where fld.tabid = ? and fld.uitype in (".generateQuestionMarks($uitypeFields).")", array($moduleid, $uitypeFields)); //crmv@42752
					if ($res) {
						while ($row = $adb->FetchByAssoc($res, -1, false)) {
							$uitype = $row['uitype'];
							foreach ($uitypeRelFiltered[$uitype] as $relmod) {
								$newrel = new ModuleRelation($module, $relmod, ModuleRelation::$TYPE_NTO1);
								$newrel->fieldid = $row['fieldid'];
								$newrel->fieldname = $row['fieldname'];
								$newrel->fieldtable = $row['tablename'];
								$newrel->fieldcolumn = $row['columnname'];
								$relations[] = $newrel;
								if ($saveCache) $this->addRelationToCache($newrel);
							}
						}
					}
				}
			}
		}

		// 1-N
		if ($type & ModuleRelation::$TYPE_1TON) {
			if ($this->hasCachedRelation($module, ModuleRelation::$TYPE_1TON)) {
				$relations = array_merge($relations, $this->getCachedRelations($module, ModuleRelation::$TYPE_1TON, $relmodules, $excludeModules));
			} else {
				if ($saveCache) $this->initializeCache($module, ModuleRelation::$TYPE_1TON);
				$query = "select fmr.fieldid,fmr.module, f.fieldname, f.tablename, f.columnname from {$table_prefix}_fieldmodulerel fmr inner join {$table_prefix}_field f on f.fieldid = fmr.fieldid where fmr.relmodule = ?"; //crmv@42752
				$params = array($module);
				if (!empty($relmodules)) {
					$query .= " and fmr.module in (".generateQuestionMarks($relmodules).")";
					$params[] = $relmodules;
				}
				if (!empty($excludeModules)) {
					$query .= " and fmr.module not in (".generateQuestionMarks($excludeModules).")";
					$params[] = $excludeModules;
				}
				$res = $adb->pquery($query, $params);
				if ($res) {
					while ($row = $adb->FetchByAssoc($res, -1, false)) {
						$newrel = new ModuleRelation($module, $row['module'], ModuleRelation::$TYPE_1TON);
						$newrel->fieldid = $row['fieldid'];
						$newrel->fieldname = $row['fieldname'];
						$newrel->fieldtable = $row['tablename'];
						$newrel->fieldcolumn = $row['columnname'];
						// crmv@43864 - search for a related
						$relres = $adb->pquery("select relation_id from {$table_prefix}_relatedlists where tabid = ? and related_tabid = ?", array(getTabid($module), getTabid($row['module'])));
						if ($relres && $adb->num_rows($relres) == 1) {
							$newrel->relationid = $adb->query_result_no_html($relres, 0, 'relation_id');
						}
						// crmv@43864e
						$relations[] = $newrel;
						if ($saveCache) $this->addRelationToCache($newrel);
					}
				}

				// special uitypes, 1-N
				$uitypeFields = array();
				foreach ($this->uitype_rel as $uitype=>$listmod) {
					if (in_array($module, $listmod)) $uitypeFields[] = $uitype;
				}
				if (count($uitypeFields) > 0) {
					$query = "select fld.fieldid, fld.tabid,fld.fieldname,fld.tablename,fld.columnname, tab.name from {$table_prefix}_field fld inner join {$table_prefix}_tab tab on tab.tabid = fld.tabid where fld.uitype in (".generateQuestionMarks($uitypeFields).")"; //crmv@42752
					$params = array($uitypeFields);
					if (!empty($relmodules)) {
						$query .= " and tab.name in (".generateQuestionMarks($relmodules).")";
						$params[] = $relmodules;
					}
					if (!empty($excludeModules)) {
						$query .= " and tab.name not in (".generateQuestionMarks($excludeModules).")";
						$params[] = $excludeModules;
					}
					$res = $adb->pquery($query, $params);
					if ($res) {
						while ($row = $adb->FetchByAssoc($res, -1, false)) {
							$newrel = new ModuleRelation($module, $row['name'], ModuleRelation::$TYPE_1TON);
							$newrel->fieldid = $row['fieldid'];
							$newrel->fieldname = $row['fieldname'];
							$newrel->fieldtable = $row['tablename'];
							$newrel->fieldcolumn = $row['columnname'];
							$relations[] = $newrel;
							if ($saveCache) $this->addRelationToCache($newrel);
						}
					}
				}
			}
		}

		// N-N - MOLTO BETA!!
		// 1. usare relatedlists e vedere se ci sono 2 related (nelle 2 direzioni)
		// 2. vedere se ci sono relatedlist che usano le funzioni N-N (get_related_list-> per ora cablate!! ARGH)
		// 3. aggiungere relazioni per i moduli con prodotti
		if ($type & ModuleRelation::$TYPE_NTON) {
			if ($this->hasCachedRelation($module, ModuleRelation::$TYPE_NTON)) {
				$relations = array_merge($relations, $this->getCachedRelations($module, ModuleRelation::$TYPE_NTON, $relmodules, $excludeModules));
			} else {
				if ($saveCache) $this->initializeCache($module, ModuleRelation::$TYPE_NTON);

				$found_relid = array();
				$query = "
					SELECT r1.relation_id, r2.relation_id as relation_id2, tab1.name as mod1, tab2.name as mod2
					FROM {$table_prefix}_relatedlists r1
						INNER JOIN {$table_prefix}_relatedlists r2 on r2.tabid = r1.related_tabid and r2.related_tabid = r1.tabid
						INNER JOIN {$table_prefix}_tab tab1 on tab1.tabid = r1.tabid
						INNER JOIN {$table_prefix}_tab tab2 on tab2.tabid = r1.related_tabid
					WHERE r1.tabid = ?"; // AND r1.tabid != r1.related_tabid"; // crmv@43611
				$params = array($moduleid);
				if (!empty($relmodules)) {
					$query .= " and tab2.name in (".generateQuestionMarks($relmodules).")";
					$params[] = $relmodules;
				}
				if (!empty($excludeModules)) {
					$query .= " and tab2.name not in (".generateQuestionMarks($excludeModules).")";
					$params[] = $excludeModules;
				}
				$res = $adb->pquery($query, $params);
				if ($res) {
					while ($row = $adb->FetchByAssoc($res, -1, false)) {
						$newrel = new ModuleRelation($module, $row['mod2'], ModuleRelation::$TYPE_NTON);
						$newrel->relationid = $row['relation_id'];
						$newrel->relationinfo = $newrel->getNtoNinfo(); //crmv@47905
						$relations[] = $newrel;
						$found_relid[$row['relation_id']] = $row['relation_id'];
						$found_relid[$row['relation_id2']] = $row['relation_id2'];
						if ($saveCache) $this->addRelationToCache($newrel);
					}
				}

				//2.1
				$ntonFunctions = array('get_related_list', 'get_messages_list', 'get_documents_dependents_list', 'get_attachments', 'get_campaigns_newsletter', 'get_newsletter_emails', 'get_faxes', 'get_sms'); // crmv@38798 crmv@43765
				$query = "
					SELECT r.relation_id, tab1.name as mod1, tab2.name as mod2
						FROM {$table_prefix}_relatedlists r
						INNER JOIN {$table_prefix}_tab tab1 on tab1.tabid = r.tabid
						INNER JOIN {$table_prefix}_tab tab2 on tab2.tabid = r.related_tabid
					WHERE r.related_tabid = ? and r.tabid != r.related_tabid and r.name in (".generateQuestionMarks($ntonFunctions).")";
				$params = array($moduleid, $ntonFunctions);
				if (!empty($found_relid)) {
					$query .= " and r.relation_id not in (".generateQuestionMarks($found_relid).")";
					$params[] = $found_relid;
				}
				if (!empty($relmodules)) {
					$query .= " and tab1.name in (".generateQuestionMarks($relmodules).")";	// crmv@43611
					$params[] = $relmodules;
				}
				if (!empty($excludeModules)) {
					$query .= " and tab1.name not in (".generateQuestionMarks($excludeModules).")"; // crmv@43611
					$params[] = $excludeModules;
				}
				$res = $adb->pquery($query, $params);
				if ($res) {
					while ($row = $adb->FetchByAssoc($res, -1, false)) {
						$newrel = new ModuleRelation($module, $row['mod1'], ModuleRelation::$TYPE_NTON);
						$newrel->relationid = $row['relation_id'];
						$relations[] = $newrel;
						$found_relid[$row['relation_id']] = $row['relation_id'];
						if ($saveCache) $this->addRelationToCache($newrel);
					}
				}

				//2.2
				$query = "
					SELECT r.relation_id, tab1.name as mod1, tab2.name as mod2
						FROM {$table_prefix}_relatedlists r
						INNER JOIN {$table_prefix}_tab tab1 on tab1.tabid = r.tabid
						INNER JOIN {$table_prefix}_tab tab2 on tab2.tabid = r.related_tabid
					WHERE r.tabid = ? and r.tabid != r.related_tabid and r.name in (".generateQuestionMarks($ntonFunctions).")";
				$params = array($moduleid, $ntonFunctions);
				if (!empty($found_relid)) {
					$query .= " and r.relation_id not in (".generateQuestionMarks($found_relid).")";
					$params[] = $found_relid;
				}
				if (!empty($relmodules)) {
					$query .= " and tab2.name in (".generateQuestionMarks($relmodules).")"; // crmv@43611
					$params[] = $relmodules;
				}
				if (!empty($excludeModules)) {
					$query .= " and tab2.name not in (".generateQuestionMarks($excludeModules).")"; // crmv@43611
					$params[] = $excludeModules;
				}
				$res = $adb->pquery($query, $params);
				if ($res) {
					while ($row = $adb->FetchByAssoc($res, -1, false)) {
						$newrel = new ModuleRelation($module, $row['mod2'], ModuleRelation::$TYPE_NTON);
						$newrel->relationid = $row['relation_id'];
						$relations[] = $newrel;
						if ($saveCache) $this->addRelationToCache($newrel);
					}
				}

				//3.1
				if (isInventoryModule($module)) {
					if ((empty($relmodules) || in_array('Products', $relmodules)) && !in_array('Products', $excludeModules)) {
						$newrel = new ModuleRelation($module, 'Products', ModuleRelation::$TYPE_NTON);
						$relations[] = $newrel;
						if ($saveCache) $this->addRelationToCache($newrel);
					}
					// crmv@38798
					if ((empty($relmodules) || in_array('Services', $relmodules)) && !in_array('Services', $excludeModules)) {
						$newrel = new ModuleRelation($module, 'Services', ModuleRelation::$TYPE_NTON);
						$relations[] = $newrel;
						if ($saveCache) $this->addRelationToCache($newrel);
					}
					// crmv@38798e

				}

				// 3.2
				if (in_array($module, array('Products', 'Services'))) { // crmv@38798
					$imods = getInventoryModules();
					if (!empty($relmodules)) $imods = array_intersect($imods, $relmodules);
					$imods = array_diff($imods, $excludeModules);
					if (!empty($imods)) {
						//crmv@3086m
						$relation_ids = array();
						$query = "
							SELECT r.relation_id, tab1.name as mod1, tab2.name as mod2
								FROM {$table_prefix}_relatedlists r
								INNER JOIN {$table_prefix}_tab tab1 on tab1.tabid = r.tabid
								INNER JOIN {$table_prefix}_tab tab2 on tab2.tabid = r.related_tabid
							WHERE r.tabid = ? and tab2.name in (".generateQuestionMarks($imods).")";
						$params = array($moduleid, $imods);
						$res = $adb->pquery($query, $params);
						if ($res) {
							while ($row = $adb->FetchByAssoc($res, -1, false)) {
								$relation_ids[$row['mod2']] = $row['relation_id'];
							}
						}
						//crmv@3086me
						foreach ($imods as $imod) {
							$newrel = new ModuleRelation($module, $imod, ModuleRelation::$TYPE_NTON);
							if (!empty($relation_ids[$imod])) $newrel->relationid = $relation_ids[$imod];	//crmv@3086m
							$relations[] = $newrel;
							if ($saveCache) $this->addRelationToCache($newrel);
						}
					}
				}
			}
		}

		return $relations;
	}

	function getRelatedModules($module) {
		$modrel = $this->getRelations($module);

		$ret = array();
		if (is_array($modrel)) {
			foreach ($modrel as $relobj) {
				$ret[$relobj->getSecondModule()] = true;
			}
			$ret = array_keys($ret);
		}

		return $ret;
	}

	// true if module1 is somehow related to module2
	function isModuleRelated($module1, $module2) {
		return in_array($module1, $this->getRelatedModules($module2));
	}

	// return a list of all related records of a record
	// return: array( id=>array('setype'=>type, 'field'=>fieldid, 'related'=>array(relid,...) ... )
	// TODO: gestire profondità maggiore di 1 (record collegato a x -> collegato a... )
	function getRelatedIds($module, $crmid, $relmodules = array(), $excludeMods = array(), $recursive = false, $groupByModule = false) {
		global $adb, $table_prefix;

		$ret = array();
		if (!is_array($relmodules)) $relmodules = array($relmodules);
		$relmodules = array_filter($relmodules);

		// get relations filtered by module
		$relations = $this->getRelations($module, null, $relmodules, $excludeMods);

		if (is_array($relations)) {
			foreach ($relations as $relation) {
				$ids = $relation->getRelatedIds($crmid);
				if (!empty($ids)) {
					$relation->getSecondModule();
					if ($groupByModule) {
						if (empty($ret[$relation->getSecondModule()])) {
							$ret[$relation->getSecondModule()] = $ids;
						} else {
							$ret[$relation->getSecondModule()] = array_merge($ret[$relation->getSecondModule()], $ids);
						}
						$ret[$relation->getSecondModule()] = array_unique($ret[$relation->getSecondModule()]);
					} else {
						$ret = array_merge($ret, $ids);
					}
				}
			}
		}
		if ($groupByModule) {
			return $ret;
		} else {
			return array_unique($ret);
		}
	}

	function countRelatedIds($module, $crmid, $relmodules = array(), $excludeMods = array(), $recursive = false) {
		global $adb, $table_prefix;

		$ret = array();
		if (!is_array($relmodules)) $relmodules = array($relmodules);
		$relmodules = array_filter($relmodules);

		// get relations filtered by module
		$relations = $this->getRelations($module, null, $relmodules, $excludeMods);

		if (is_array($relations)) {
			foreach ($relations as $relation) {
				$cnt = $relation->countRelatedIds($crmid);
				$ret[$relation->getSecondModule()] = $cnt;
			}
		}

		return $ret;
	}

	//crmv@43864 crmv@3086m
	function getTurboliftRelations($focus, $module, $crmid, $relmodules = array(), $excludeMods = array(), $recursive = false) {
		global $adb, $table_prefix;

		$relations = $pins = array();
		if (!is_array($relmodules)) $relmodules = array($relmodules);
		$relmodules = array_filter($relmodules);

		$CRMVUtils = CRMVUtils::getInstance();
		 //TODO:per relationid
		$pinRelatedLists = $CRMVUtils->getPinRelatedLists(getTabid($module));

		// get related lists
		$related_array = getRelatedLists($module,$focus);
		if (!empty($related_array)) {
			$related_array_info = array();
			foreach ($related_array as $header => $info) {
				$related_array_info[$info['relationId']] = array(
					'header'=>$header,
					'related_module' => $info['related_tabname'],
					'relationId'=>$info['relationId'],
					'actions'=>$info['actions'],
					'name'=>$info['name'],
				);
				if (file_exists('themes/images/modulesimg/'.$info['related_tabname'].'.png')) {
					$related_array_info[$info['relationId']]['img'] = 'themes/images/modulesimg/'.$info['related_tabname'].'.png';
				}
			}
		}

		$relations1N = $this->getRelations($module, ModuleRelation::$TYPE_1TON, $relmodules, $excludeMods);
		$relationsNN = $this->getRelations($module, ModuleRelation::$TYPE_NTON, $relmodules, $excludeMods);
		$rl = array();
		if (is_array($relations1N)) {
			foreach ($relations1N as $relation) {
				$relationid = (empty($relation->relationid) ? 'fld_'.$relation->fieldid : $relation->relationid);
				$ids = $relation->getRelatedIds($crmid);
				if (!empty($ids)) {
					$rl[$relationid] = array('secmod'=>$relation->getSecondModule(), 'ids'=>$ids);
				}
			}
		}
		if (is_array($relationsNN)) {
			foreach ($relationsNN as $relation) {
				$relationid = $relation->relationid;
				$ids = $relation->getRelatedIds($crmid);
				if (!empty($ids)) {
					if (empty($rl[$relationid])) {
						$rl[$relationid] = array('secmod'=>$relation->getSecondModule(), 'ids'=>$ids);
					} else {
						$rl[$relationid]['ids'] = array_merge($rl[$relationid]['ids'], $ids);
					}
				}
			}
		}
		$force_related = array('ChangeLog','Fax','Sms');
		foreach ($rl as $relationid => $relinfo) {
			$mod = $relinfo['secmod'];
			$ids = array_unique($relinfo['ids']);
			// TODO: services??
			if (isInventoryModule($module) && in_array($mod,array('Products'))) {
				continue;
			}
//			if ($mod == 'Calendar') {
//				$result = $adb->pquery("select activitytype from {$table_prefix}_activity WHERE activityid = ?",array());
//			}
			if ($mod == 'Events') {
				continue;
			}
			$pin = false;
			$display = 'block';
			if (!empty($pinRelatedLists[$mod])) {
				$pins[$pinRelatedLists[$mod]] = $related_array[$pinRelatedLists[$mod]];
				$pin = true;
				$display = 'none';
			}
			$tmp = array(
				'type'=>'other',
				'display'=>$display,
				'module'=>$mod,
				'count'=>count($ids)
			);
			if (!empty($related_array_info[$relationid])) {
				$tmp = array_merge($tmp,$related_array_info[$relationid]);
				unset($related_array_info[$relationid]);
			}
			if (empty($tmp['relationId'])) continue;
			$relations[] = $tmp;
		}
		if (!empty($related_array_info)) {
			foreach($related_array_info as $relationid => $info) {
				$mod = $info['related_module'];
				if ($module == 'Calendar' && in_array($mod,array('','Contacts'))) {	// remove related list of Users and Contacts
					continue;
				}
				$tmp = array(
					'type'=>'other',
					'display'=>'none',
					'module'=>$mod
				);
				$tmp = array_merge($tmp,$related_array_info[$relationid]);
				$relations[] = $tmp;
			}
		}
		return array('turbolift'=>$relations,'pin'=>$pins);
	}
	//crmv@43864e crmv@3086me

	//crmv@44609
	function relate($module1, $record1, $module2, $record2) {
		if (!is_array($record2)) $record2 = array($record2);
		if (empty($record2)) return false;
		
		if ($this->isModuleRelated($module1, $module2)) {
			$moduleRelations = $this->getRelations($module1, null, $module2);
			if (!empty($moduleRelations)) {
				foreach($moduleRelations as $moduleRelation) {
					$moduleRelationType = $moduleRelation->getType();
					if ($moduleRelationType == ModuleRelation::$TYPE_1TO1) {
						// TODO
					} elseif ($moduleRelationType == ModuleRelation::$TYPE_1TON) {
						foreach ($record2 as $r) {
							$focus = CRMEntity::getInstance($module2);
							$focus->retrieve_entity_info($r, $module2);
							$focus->mode = 'edit';
							$focus->id = $r;
							$focus->column_fields[$moduleRelation->fieldname] = $record1;
							$focus->save($module2);
						}
					} elseif ($moduleRelationType == ModuleRelation::$TYPE_NTO1) {
						$focus = CRMEntity::getInstance($module1);
						$focus->retrieve_entity_info($record1, $module1);
						$focus->mode = 'edit';
						$focus->id = $record1;
						$focus->column_fields[$moduleRelation->fieldname] = $record2[0];
						$focus->save($module1);
					} elseif ($moduleRelationType == ModuleRelation::$TYPE_NTON) {
						$focus = CRMEntity::getInstance($module1);
						$focus->save_related_module($module1, $record1, $module2, $record2);
					}
					break;
				}
			}
		}
	}
	//crmv@44609e
}
?>