<?php
/***************************************************************************************
 * The contents of this file are subject to the CRMVILLAGE.BIZ VTECRM License Agreement
 * ("licenza.txt"); You may not use this file except in compliance with the License
 * The Original Code is:  CRMVILLAGE.BIZ VTECRM
 * The Initial Developer of the Original Code is CRMVILLAGE.BIZ.
 * Portions created by CRMVILLAGE.BIZ are Copyright (C) CRMVILLAGE.BIZ.
 * All Rights Reserved.
 ***************************************************************************************/
//crmv@37004
require_once('modules/Messages/MessagesUtilities.php');

class MessagesRelationManager extends MessagesUtilities {

	var $peopleModules = array('Leads', 'Contacts', 'Accounts', 'Vendors');

	function __construct() {
		parent::__construct();
		global $table_prefix;
		$this->relation_table = "{$table_prefix}_messagesrel";
		$this->relation_table_id = 'messagehash';
		$this->relation_table_otherid = 'crmid';
		$this->relation_table_module = '';
		$this->relation_table_othermodule = 'module';
	}

	// insert into the relation table
	function save_related_module($module, $crmid, $with_module, $with_crmid) {
		global $adb, $table_prefix;

		$thistabid = getTabId('Messages');
		$othertabid = getTabId($with_module);

		$hasrelation = false;
		$result = $adb->pquery("SELECT * FROM {$table_prefix}_relatedlists WHERE (tabid = ? AND related_tabid = ?)", array($othertabid,$thistabid));
		if ($result && $adb->num_rows($result) > 0) $hasrelation = true;

		if ($hasrelation) {
			if(!is_array($with_crmid)) $with_crmid = Array($with_crmid);

			$this->column_fields['messageid'] = getSingleFieldValue($this->table_name, 'messageid', $this->table_index, $crmid);
			$hash = $this->getMessageHash();

			if ($hash) {
				foreach ($with_crmid as $relcrmid) {
					$checkpresence = $adb->pquery("SELECT crmid FROM {$this->relation_table} WHERE {$this->relation_table_id} = ? AND crmid = ? AND module = ?", Array($hash, $relcrmid, $with_module));
					if ($checkpresence && $adb->num_rows($checkpresence) > 0) continue;
					$adb->pquery("INSERT INTO {$this->relation_table} ({$this->relation_table_id}, crmid, module) VALUES (?,?,?)", array($hash, $relcrmid, $with_module));
				}
			}
		}
	}

	function save_related_module_small($messageid, $with_module, $with_crmid) {
		global $adb;

		if(!is_array($with_crmid)) $with_crmid = Array($with_crmid);

		$hash = $this->getMessageHash($messageid);

		if ($hash) {
			foreach ($with_crmid as $relcrmid) {
				$checkpresence = $adb->pquery("SELECT crmid FROM {$this->relation_table} WHERE {$this->relation_table_id} = ? AND crmid = ? AND module = ?", Array($hash, $relcrmid, $with_module));
				if ($checkpresence && $adb->num_rows($checkpresence) > 0) continue;
				$adb->pquery("INSERT INTO {$this->relation_table} ({$this->relation_table_id}, crmid, module) VALUES (?,?,?)", array($hash, $relcrmid, $with_module));
			}
		}
	}

	/** Function to unlink an entity with given Id from another entity */
	function unlinkRelationship($id, $return_module, $return_id) {
		global $adb, $table_prefix;

		$mhash = getSingleFieldValue($this->table_name, 'messagehash', $this->table_index, $id);
		if ($mhash) {
			$query = "DELETE FROM {$this->relation_table} WHERE ({$this->relation_table_id} = ? AND crmid=?)";
			$params = array($mhash, $return_id);
			$r = $this->db->pquery($query, $params);
		}
	}

	function haveRelations($id,$modules='',$skip_modules='',$thread='') {
		//TODO do a unique query and cache values
		global $adb,$table_prefix;
		if (empty($skip_modules)) {
			$skip_modules = 'ModComments';
		}
		$query = "select {$this->relation_table}.{$this->relation_table_otherid}
				from {$this->relation_table}
				inner join {$table_prefix}_crmentity ON {$table_prefix}_crmentity.crmid = {$this->relation_table}.{$this->relation_table_otherid}
				where deleted = 0";
		if (!empty($thread)) {
			$children = $this->getChildren($thread,'',false,'distinct messageSon.messagehash');
			$query .= " and {$this->relation_table_id} in (".generateQuestionMarks($children).")";
			$params = array($children);
		} else {
			$mhash = getSingleFieldValue($this->table_name, 'messagehash', $this->table_index, $id);
			$query .= " and {$this->relation_table_id} = ?";
			$params = array($mhash);
		}
		if (!empty($modules)) {
			if (!is_array($modules)) {
				$modules = array($modules);
			}
			$query .= " and $this->relation_table_othermodule in (".generateQuestionMarks($modules).")";
			$params[] = $modules;
			$skip_modules = '';
		}
		if (!empty($skip_modules)) {
			if (!is_array($skip_modules)) {
				$skip_modules = array($skip_modules);
			}
			$query .= " and $this->relation_table_othermodule not in (".generateQuestionMarks($skip_modules).")";
			$params[] = $skip_modules;
		}
		$result = $adb->pquery($query,$params);
		if ($result && $adb->num_rows($result) > 0) {
			return true;
		}
		return false;
	}

	// calculate hash for the current message
	// this hash must be unique for every message
	function getMessageHash($messageid = null) {
		$mid = ($messageid ? $messageid : $this->column_fields['messageid']);
		if (!empty($mid))
			return sha1($mid);
		else
			return '';
	}

	// rebuild hashes for all messages
	function rebuildAllHashes() {
		global $adb, $table_prefix;
		if ($adb->isMysql()) {
			// optimized query for mysql
			$res = $adb->query("update {$this->table_name} inner join {$table_prefix}_crmentity on {$table_prefix}_crmentity.crmid = {$this->table_name}.{$this->table_index} set messagehash = SHA1(messageid) where {$table_prefix}_crmentity.deleted = 0");
		} else {
			// general query
			$res = $adb->query("select {$this->table_index}, messageid from {$this->table_name} inner join {$table_prefix}_crmentity on {$table_prefix}_crmentity.crmid = {$this->table_name}.{$this->table_index} where {$table_prefix}_crmentity.deleted = 0");
			if ($res) {
				while ($row = $adb->FetchByAssoc($res, -1, false)) {
					$adb->pquery("update {$this->table_name} set messagehash = ? where {$this->table_index} = ?", array($this->getMessageHash($row['messageid']), $row[$this->table_index]));
				}
			}
		}
	}

	// parse an email address and retrieve all entities with the specified email
	// returns: array("module"=>array(id1, id2, ...), ... )
	// TODO: order records by relevance, or put second level stuff in a different array
	function getEntitiesFromEmail($email, $searchDomain=true, $deepSearch=true, $modules=null, $return_first_result=false) {
		global $current_user, $adb, $table_prefix;
		$email = trim($email);

		//crmv@41883
		require_once('include/utils/EmailDirectory.php');
		$emailDirectory = new EmailDirectory();
		if ($return_first_result) {
			$record = $emailDirectory->getRecord($email);
			if (!empty($record)) {
				return $record;
			}
		}
		//crmv@41883e

		// modules to search for email address
		if (empty($modules)) {
			$modules = array('Leads', 'Accounts', 'Contacts', 'Vendors', 'Users');
		}
		// modules to search by email domain
		$modulesByDomain = array('Accounts');
		// email domains excluded by search by domain, regexp
		$excludedDomains = array(
			'google', 'gmail', 'yahoo', 'hotmail', 'aol', 'msn', 'hotmail', 'aim\.', 'zoho\.com', 'outlook\.com', 'live\.com',
			'alice\.it', 'vodafone\.it', 'tiscali\.it', 'infostrada\.it', 'libero\.it', 'wind\.it', 'tele2\.it', 'fastweb', 'virgilio\.it', 'aruba\.it', 'mail\.it',
		);

		// email uitypes
		$emailUitype = $emailDirectory->getUItypes();

		$emailDomain = preg_replace('/^.*@/', '', $email);
		$emailSearchDomain = $emailDomain;

		// check for valid search domain
		foreach ($excludedDomains as $ed) {
			if (preg_match("/$ed/i", $emailSearchDomain)) {
				$emailSearchDomain = '';
				break;
			}
		}

		// get all email fields by module
		$emailFields = array();
		$query = "select fieldid, tablename, columnname, fieldname, {$table_prefix}_tab.name
					from {$table_prefix}_field
					inner join {$table_prefix}_tab ON {$table_prefix}_tab.tabid = {$table_prefix}_field.tabid
					where {$table_prefix}_tab.name in (".generateQuestionMarks($modules).") and uitype in (".generateQuestionMarks($emailUitype).")";
		$res = $adb->pquery($query, array($modules, $emailUitype));
		if ($res) {
			while ($row = $adb->FetchByAssoc($res, -1, false)) {
				$emailFields[$row['name']][] = $row;
			}
		}

		// generate query to retrieve ids
		$relatedIds = array();
		foreach ($modules as $module) {
			$moduleFields = $emailFields[$module];
			if (empty($moduleFields)) continue;

			$qg = QueryGenerator::getInstance($module, $current_user);
			$accessibleFieldList = array_keys($qg->getModuleFields());

			$fields = array();
			foreach ($moduleFields as $i => $fieldinfo) {
				if ($module == 'Users' || in_array($fieldinfo['fieldname'],$accessibleFieldList)) {
					$fields[] = $fieldinfo['fieldname'];
				} else {
					unset($moduleFields[$i]);
				}
			}
			$qg->setFields($fields);

			if ($module == 'Users') {
				$sql = $qg->getQuery();
				// add condition
				$conditions = array();
				foreach ($moduleFields as $fieldinfo) {
					$conditions[] = $fieldinfo['tablename'].".".$fieldinfo['columnname']." = '$email'";
					// search also by domain
					if ($searchDomain && !empty($emailSearchDomain) && in_array($module, $modulesByDomain)) {
						$conditions[] = $fieldinfo['tablename'].".".$fieldinfo['columnname']." like '%@$emailSearchDomain'";
					}
				}
				$sql .= ' AND ('.implode(' OR ', $conditions).')';
			} else {
				$qg->startGroup('');
				foreach ($moduleFields as $index => $fieldinfo) {
					if($index > 0) {
						$qg->addConditionGlue(QueryGenerator::$OR);
					}
					$qg->addCondition($fieldinfo['fieldname'],$email,'e');
					// search also by domain
					if ($searchDomain && !empty($emailSearchDomain) && in_array($module, $modulesByDomain)) {
						$qg->addConditionGlue(QueryGenerator::$OR);
						$qg->addCondition($fieldinfo['fieldname'],"@$emailSearchDomain",'ew');
					}
				}
				$qg->endGroup();
				$sql = $qg->getQuery();
			}

			// add crmid
			if ($module == 'Users') {
				$sql = preg_replace('/^select.*?from/i', "select {$table_prefix}_users.id as crmid from", $sql);
			} else {
				$sql = preg_replace('/^select.*?from/i', "select {$table_prefix}_crmentity.crmid from", $sql);
			}

			// get the result
			if ($return_first_result) {
				$res = $adb->limitQuery($sql,0,1);
			} else {
				$res = $adb->query($sql);
			}
			if ($res) {
				while ($row = $adb->FetchByAssoc($res, -1, false)) {
					if ($return_first_result) {
						$emailDirectory->create($email,$row['crmid'],$module);	//crmv@41883
						return array('module'=>$module,'crmid'=>$row['crmid']);
					}
					$relatedIds[$module][] = $row['crmid'];
				}
			}
			if (count($relatedIds[$module]) > 0) {
				$relatedIds[$module] = array_unique($relatedIds[$module]);
			}
		}

		$rm = RelationManager::getInstance();

		// crmv@42752
		// retrieve accounts for contacts found
		if ($deepSearch && is_array($relatedIds['Contacts']) && count($relatedIds['Contacts']) > 0) {
			foreach ($relatedIds['Contacts'] as $contid) {
				$linkedacc = $rm->getRelatedIds('Contacts', $contid, 'Accounts');
				if (!empty($linkedacc)) {
					if (!empty($relatedIds['Accounts'])) {
						$relatedIds['Accounts'] = array_merge($relatedIds['Accounts'], $linkedacc);
					} else {
						$relatedIds['Accounts'] = $linkedacc;
					}
				}
			}
			if (!empty($relatedIds['Accounts'])) {
				$relatedIds['Accounts'] = array_unique($relatedIds['Accounts']);
			}
		}
		// crmv@42752e

		// now for every account, retrieve other contacts
		if ($deepSearch && is_array($relatedIds['Accounts']) && count($relatedIds['Accounts']) > 0) {
			foreach ($relatedIds['Accounts'] as $accid) {
				$linkedcont = $rm->getRelatedIds('Accounts', $accid, 'Contacts');
				if (!empty($linkedcont)) {
					if (!empty($relatedIds['Contacts'])) {
						$relatedIds['Contacts'] = array_merge($relatedIds['Contacts'], $linkedcont);
					} else {
						$relatedIds['Contacts'] = $linkedcont;
					}
				}
			}
			if (!empty($relatedIds['Contacts'])) {
				$relatedIds['Contacts'] = array_unique($relatedIds['Contacts']);
			}
		}

		//crmv@41883 : salvo anche se non ho trovato niente cos� la volta prossima so che non c'� nulla
		if ($return_first_result && empty($relatedIds)) {
			$emailDirectory->create($email);
		}
		//crmv@41883e

		return $relatedIds;
	}

	// get modules for linking
	function getPopupLinkModules($mode='') {
		global $adb, $table_prefix;

		$rm = RelationManager::getInstance();

		$excludeModules = array(
			'Documents', 'ModComments', //crmv@42752
		);

		$mods = $rm->getRelatedModules('Messages');
		$mods = array_diff($mods, $excludeModules);
		if ($mode == 'linkdocument') {
			$mods_docs = $rm->getRelatedModules('Documents');
			$mods = array_intersect($mods, $mods_docs);
		}
		// permission cycle
		foreach ($mods as $k=>$mod) {
			if (isPermitted($mod, 'EditView') != 'yes') unset($mods[$k]);
		}

		// sort by module label
		usort($mods, create_function('$m1, $m2', 'return strcasecmp(getTranslatedString($m1, $m1), getTranslatedString($m2, $m2));'));

		return $mods;
	}

	// get modules for creation
	function getPopupCreateModules($mode='') {

		$excludeModules = array_merge(array('Calendar'), getInventoryModules()); // crmv@42752

		$mods = $this->getPopupLinkModules($mode);
		$mods = array_diff($mods, $excludeModules);

		return $mods;
	}

	// returns an array of fields populated with relevant values for the related module
	function getPopupQCreateValues($module, $relatedIds, $email = '', $name = '') {
		global $adb, $table_prefix;

		$ret = array();

		// calculate name fields
		if (!empty($name)) {
			$name_arr = explode(' ',$name);
			$lenght = count($name_arr);
			if ($lenght == 1) {
				$first_name = '';
				$last_name = $name_arr[0];
			} else {
				$first_name = $name_arr[0];
				array_shift($name_arr);
				$last_name = implode(' ',$name_arr);
			}
		}

		// get relation info (for id types)
		$rm = RelationManager::getInstance();
		$relmods = $rm->getRelations($module, ModuleRelation::$TYPE_NTO1);

		// filter based on related mods
		foreach ($relmods as $k=>$rel) {
			$destmod = $rel->getSecondModule();
			if (!array_key_exists($destmod, $relatedIds)) {
				//unset($relmods[$k]);
			}
		}

		$tabid = getTabid($module);
		// take all the quickcreate fields, regardless of the user permission (i will be checked in the QuicCreate function)
		$res = $adb->pquery("select fieldname, uitype from {$table_prefix}_field where quickcreate in (0,2) and tabid = ? and {$table_prefix}_field.presence in (0,2) and displaytype != 2", array($tabid));

		if ($res && $adb->num_rows($res) > 0) {
			while ($row = $adb->FetchByAssoc($res, -1, false)) {
				$value = '';
				$fieldname = $row['fieldname'];
				// check if it's a relation field (take the first one)
				foreach ($relmods as $k=>$rel) {
					if ($rel->fieldname == $fieldname) {
						$value = $relatedIds[$rel->getSecondModule()][0];
						break;
					}
				}
				// check email fields
				switch ($row['uitype']) {
					case '13': {
						$value = $email;
					}
				}
				// populate name fields
				if (!empty($name) && in_array($module,$this->peopleModules)) {
					switch ($module) {
						case 'Leads':
						case 'Contacts':
							if (!empty($first_name) && $fieldname == 'firstname') {
								$value = $first_name;
							}
							if (!empty($last_name) && $fieldname == 'lastname') {
								$value = $last_name;
							}
							break;
						case 'Accounts':
							if ($fieldname == 'accountname') {
								$value = $name;
							}
							break;
						case 'Vendors':
							if ($fieldname == 'vendorname') {
								$value = $name;
							}
							break;
					}
				}
				// set the value
				if (!empty($value)) $ret[$fieldname] = $value;
			}
		}
		return $ret;
	}
	//crmv@44610
	function getMessagesWithSameHash($id) {
		global $adb, $table_prefix;
		$query = "select {$table_prefix}_messages.messagesid
					from {$table_prefix}_messages
					inner join {$table_prefix}_crmentity ON {$table_prefix}_crmentity.crmid = {$table_prefix}_messages.messagesid
					where deleted = 0";
		$mhash = getSingleFieldValue($this->table_name, 'messagehash', $this->table_index, $id);
		$query .= " and messagehash = ?";
		$params = array($mhash);
		$return = array();
		$result = $adb->pquery($query,$params);
		if ($result && $adb->num_rows($result) > 0) {
			while($row=$adb->fetchByASsoc($result)) {
				$return[] = $row['messagesid'];
			}
		}
		return $return;
	}
	//crmv@44610e	
}
?>